@@ -1549,7 +1549,7 @@ void stress_rndstr(char *str, size_t len)
 	 * Be careful if expanding this alphabet, some of this function's users
 	 * use it to generate random filenames.
 	 */
-	static const char alphabet[64] = 
+	static const char alphabet[64] =
 		"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 		"abcdefghijklmnopqrstuvwxyz"
 		"0123456789" "-_";
@@ -2308,6 +2308,10 @@ const char *stress_get_compiler(void)
       defined(__INTEL_COMPILER) &&	\
       defined(__INTEL_COMPILER_UPDATE)
 	static const char cc[] = "icc " XSTRINGIFY(__INTEL_COMPILER) "." XSTRINGIFY(__INTEL_COMPILER_UPDATE) "";
+#elif defined(__INTEL_CLANG_COMPILER)
+	static const char cc[] = "icx " XSTRINGIFY(__INTEL_CLANG_COMPILER) "";
+#elif defined(__INTEL_LLVM_COMPILER)
+	static const char cc[] = "icx " XSTRINGIFY(__INTEL_LLVM_COMPILER) "";
 #elif defined(__TINYC__)
 	static const char cc[] = "tcc " XSTRINGIFY(__TINYC__) "";
 #elif defined(__PCC__) &&			\
@@ -3270,7 +3274,8 @@ void NORETURN MLOCKED_TEXT stress_sig_handler_exit(int signum)
  *  __stack_chk_fail()
  *	override stack smashing callback
  */
-#if (defined(__GNUC__) || defined(__clang__)) &&	\
+#if defined(__GNUC__) &&	\
+    !defined(__clang__) &&	\
     defined(HAVE_WEAK_ATTRIBUTE)
 extern void __stack_chk_fail(void);
 
