@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,6 +66,10 @@ UNEXPECTED
 #include <sys/prctl.h>
 #endif
 
+#if defined(HAVE_SYS_TIMEX_H)
+#include <sys/timex.h>
+#endif
+
 #if defined(HAVE_SYS_RANDOM_H)
 #include <sys/random.h>
 #endif
@@ -206,7 +210,7 @@ static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 
 	while (keep_stressing_flag() && (n > 0)) {
 		ssize_t ret;
-		size_t count = (size_t)STRESS_MINIMUM(n, FALLOCATE_BUF_SIZE);
+		const size_t count = (size_t)STRESS_MINIMUM(n, FALLOCATE_BUF_SIZE);
 
 		ret = write(fd, buffer, count);
 		if (ret >= 0) {
@@ -784,11 +788,11 @@ int shim_nanosleep_uint64(uint64_t nsec)
 		errno = 0;
 		if (usleep(usec) < 0) {
 			if (errno == EINTR) {
-				double t_left = t_end - stress_time_now();
+				const double t_left = t_end - stress_time_now();
 
 				if (t_left < 0.0)
 					return 0;
-				usec = (useconds_t)(t_left * 1000000.0);
+				usec = (useconds_t)(t_left * STRESS_DBL_MICROSECOND);
 				if (usec == 0)
 					return 0;
 				if (keep_stressing_flag())
@@ -1155,13 +1159,10 @@ int shim_brk(void *addr)
 #elif defined(HAVE_BRK)
 	return brk(addr);
 #else
-	uintptr_t brkaddr;
-	intptr_t inc;
-	void *newbrk;
+	const uintptr_t brkaddr = (uintptr_t)shim_sbrk(0);
+	intptr_t inc = brkaddr - (intptr_t)addr;
+	const void *newbrk = shim_sbrk(inc);
 
-	brkaddr = (uintptr_t)shim_sbrk(0);
-	inc = brkaddr - (intptr_t)addr;
-	newbrk = shim_sbrk(inc);
 	if (newbrk == (void *)-1) {
 		if (errno != ENOSYS)
 			errno = ENOMEM;
@@ -1247,16 +1248,14 @@ size_t shim_strlcat(char *dst, const char *src, size_t len)
 	register const char *s = src;
 	register size_t n = len, tmplen;
 
-	while (n-- && *d != '\0') {
+	while (n-- && *d != '\0')
 		d++;
-	}
 
 	tmplen = d - dst;
 	n = len - tmplen;
 
-	if (!n) {
+	if (!n)
 		return strlen(s) + tmplen;
-	}
 
 	while (*s != '\0') {
 		if (n != 1) {
@@ -1934,6 +1933,22 @@ int shim_clock_getres(clockid_t clk_id, struct timespec *res)
 #endif
 }
 
+/*
+ *   shim_clock_adjtime
+ *	wrapper for linux clock_adjtime system call
+ */
+int shim_clock_adjtime(clockid_t clk_id, shim_timex_t *buf)
+{
+#if defined(CLOCK_THREAD_CPUTIME_ID) && \
+    defined(HAVE_CLOCK_ADJTIME) &&	\
+    defined(HAVE_SYS_TIMEX_H) &&	\
+    defined(HAVE_TIMEX)
+	return clock_adjtime(clk_id, buf);
+#else
+	return (int)shim_enosys(0, clk_id, buf);
+#endif
+}
+
 /*
  *   shim_clock_gettime
  *	wrapper for linux clock_gettime system call,
