@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -56,9 +56,12 @@ static void MLOCKED_TEXT stress_bind_mount_child_handler(int signum)
  */
 static int stress_bind_mount_child(void *parg)
 {
-	const stress_args_t *args = ((stress_pthread_args_t *)parg)->args;
-	char path[PATH_MAX];
-	int ret;
+	stress_pthread_args_t *pargs = (stress_pthread_args_t *)parg;
+	const stress_args_t *args = pargs->args;
+	const char *path = (const char *)pargs->data;
+	double mount_duration = 0.0, umount_duration = 0.0;
+	double mount_count = 0.0, umount_count = 0.0;
+	double rate;
 
 	if (stress_sighandler(args->name, SIGALRM,
 				stress_bind_mount_child_handler, NULL) < 0) {
@@ -74,20 +77,13 @@ static int stress_bind_mount_child(void *parg)
 	}
 	stress_parent_died_alarm();
 
-	(void)stress_temp_dir(path, sizeof(path), args->name, getpid(), args->instance);
-	ret = mkdir(path, S_IRUSR | S_IRWXU);
-	if (ret < 0) {
-		(void)shim_rmdir(path);
-		pr_err("%s: mkdir %s failed, errno=%d (%s)\n",
-			args->name, path, errno, strerror(errno));
-		return EXIT_NO_RESOURCE;
-	}
-
 	do {
 		int rc, retries;
 		DIR *dir;
 		struct dirent *d;
+		double t;
 
+		t = stress_time_now();
 		rc = mount("/", path, "", MS_BIND | MS_REC | MS_RDONLY, 0);
 		if (rc < 0) {
 			if (errno != ENOSPC)
@@ -95,6 +91,8 @@ static int stress_bind_mount_child(void *parg)
 					args->name, errno, strerror(errno));
 			break;
 		}
+		mount_duration += stress_time_now() - t;
+		mount_count += 1.0;
 
 		/*
 		 *  Check if we can stat the files in the bound mount path
@@ -123,24 +121,38 @@ bind_umount:
 		for (retries = 0; retries < 15; retries++) {
 #if defined(HAVE_UMOUNT2) &&	\
     defined(MNT_DETACH)
+			t = stress_time_now();
 			rc = umount2(path, MNT_DETACH);
-			if (rc == 0)
+			if (rc == 0) {
+				umount_duration += stress_time_now() - t;
+				umount_count += 1.0;
 				break;
+			}
 			(void)shim_usleep(50000);
 #else
 			/*
 			 * The following fails with -EBUSY, but try it anyhow
 			 *  just to make the kernel work harder
 			 */
+			t = stress_time_now();
 			rc = umount(path);
-			if (rc == 0)
+			if (rc == 0) {
+				umount_duration += stress_time_now() - t;
+				umount_count += 1.0;
 				break;
+			}
 #endif
 		}
 		inc_counter(args);
 	} while (keep_stressing_flag() &&
 		 (!args->max_ops || (get_counter(args) < args->max_ops)));
 
+	rate = (mount_count > 0.0) ? (double)mount_duration / mount_count : 0.0;
+	stress_metrics_set(args, 0, "microsecs per mount", rate * STRESS_DBL_MICROSECOND);
+	rate = (umount_count > 0.0) ? (double)umount_duration / umount_count : 0.0;
+	stress_metrics_set(args, 1, "microsecs per umount", rate * STRESS_DBL_MICROSECOND);
+
+	/* Remove path in child process just in case parent fails to reap it */
 	(void)shim_rmdir(path);
 	return 0;
 }
@@ -151,11 +163,21 @@ bind_umount:
  */
 static int stress_bind_mount(const stress_args_t *args)
 {
-	int pid = 0, status;
-	stress_pthread_args_t pargs = { args, NULL, 0 };
+	int pid = 0, status, ret;
+	char path[PATH_MAX];
+	stress_pthread_args_t pargs = { args, path, 0 };
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	(void)stress_temp_dir(path, sizeof(path), args->name, getpid(), args->instance);
+	ret = mkdir(path, S_IRUSR | S_IRUSR | S_IRGRP | S_IWGRP);
+	if (ret < 0) {
+		(void)shim_rmdir(path);
+		pr_err("%s: mkdir %s failed, errno=%d (%s)\n",
+			args->name, path, errno, strerror(errno));
+		return EXIT_NO_RESOURCE;
+	}
+
 	do {
 		static char stack[CLONE_STACK_SIZE];
 		char *stack_top = (char *)stress_get_stack_top((void *)stack, CLONE_STACK_SIZE);
@@ -178,6 +200,8 @@ static int stress_bind_mount(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	(void)shim_rmdir(path);
+
 	return EXIT_SUCCESS;
 }
 
@@ -189,8 +213,9 @@ stressor_info_t stress_bind_mount_info = {
 };
 #else
 stressor_info_t stress_bind_mount_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS | CLASS_PATHOLOGICAL,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without Linux bind-mount options MS_BIND"
 };
 #endif
