@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-asm-x86.h"
 #include "core-cache.h"
 #include "core-put.h"
 
@@ -61,7 +62,6 @@ static uint64_t disabled_flags;
 
 static const stress_help_t help[] = {
 	{ "C N","cache N",	 	"start N CPU cache thrashing workers" },
-	{ NULL,	"cache-ops N",	 	"stop after N cache bogo operations" },
 #if defined(HAVE_ASM_X86_CLDEMOTE)
 	{ NULL,	"cache-cldemote",	"cache line demote (x86 only)" },
 #endif
@@ -75,13 +75,14 @@ static const stress_help_t help[] = {
 #endif
 	{ NULL,	"cache-level N",	"only exercise specified cache" },
 	{ NULL, "cache-no-affinity",	"do not change CPU affinity" },
+	{ NULL,	"cache-ops N",	 	"stop after N cache bogo operations" },
 	{ NULL,	"cache-prefetch",	"prefetch on memory reads/writes" },
 #if defined(HAVE_BUILTIN_SFENCE)
 	{ NULL,	"cache-sfence",		"serialize stores with sfence" },
 #endif
 	{ NULL,	"cache-ways N",		"only fill specified number of cache ways" },
 	{ NULL, "cache-wb",		"cache line writeback (x86 only)" },
-	{ NULL,	NULL,		 NULL }
+	{ NULL,	NULL,			NULL }
 };
 
 static int stress_cache_set_flag(const uint32_t flag)
@@ -183,41 +184,25 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 #endif
 
 #if defined(HAVE_ASM_X86_CLFLUSH)
-static inline void clflush(void *p)
-{
-        __asm__ __volatile__("clflush (%0)\n" : : "r"(p) : "memory");
-}
-#define SHIM_CLFLUSH(p)		clflush(p)
+#define SHIM_CLFLUSH(p)		stress_asm_x86_clflush(p)
 #else
 #define SHIM_CLFLUSH(p)
 #endif
 
 #if defined(HAVE_ASM_X86_CLFLUSHOPT)
-static inline void clflushopt(void *p)
-{
-        __asm__ __volatile__("clflushopt (%0)\n" : : "r"(p) : "memory");
-}
-#define SHIM_CLFLUSHOPT(p)	clflushopt(p)
+#define SHIM_CLFLUSHOPT(p)	stress_asm_x86_clflushopt(p)
 #else
 #define SHIM_CLFLUSHOPT(p)
 #endif
 
 #if defined(HAVE_ASM_X86_CLDEMOTE)
-static inline void cldemote(void *p)
-{
-        __asm__ __volatile__("cldemote (%0)\n" : : "r"(p) : "memory");
-}
-#define SHIM_CLDEMOTE(p)	cldemote(p)
+#define SHIM_CLDEMOTE(p)	stress_asm_x86_cldemote(p)
 #else
 #define SHIM_CLDEMOTE(p)
 #endif
 
 #if defined(HAVE_ASM_X86_CLWB)
-static inline void clwb(void *p)
-{
-        __asm__ __volatile__("clwb (%0)\n" : : "r"(p) : "memory");
-}
-#define SHIM_CLWB(p)		clwb(p)
+#define SHIM_CLWB(p)		stress_asm_x86_clwb(p)
 #else
 #define SHIM_CLWB(p)
 #endif
@@ -617,10 +602,9 @@ static int stress_cache(const stress_args_t *args)
 {
 #if defined(HAVE_SCHED_GETAFFINITY) &&	\
     defined(HAVE_SCHED_GETCPU)
-	cpu_set_t mask;
+	cpu_set_t proc_mask;
 	NOCLOBBER uint32_t cpu = 0;
 	const uint32_t cpus = (uint32_t)stress_get_processors_configured();
-	cpu_set_t proc_mask;
 	NOCLOBBER bool pinned = false;
 #endif
 	uint32_t cache_flags = 0;
@@ -628,7 +612,7 @@ static int stress_cache(const stress_args_t *args)
 	int ret = EXIT_SUCCESS;
 	uint8_t *const mem_cache = g_shared->mem_cache;
 	const uint64_t mem_cache_size = g_shared->mem_cache_size;
-	uint64_t i = stress_mwc64() % mem_cache_size;
+	uint64_t i = stress_mwc64modn(mem_cache_size);
 	uint64_t k = i + (mem_cache_size >> 1);
 	NOCLOBBER uint64_t r = 0;
 	uint64_t inc = (mem_cache_size >> 2) + 1;
@@ -637,7 +621,7 @@ static int stress_cache(const stress_args_t *args)
 	disabled_flags = 0;
 
 	if (sigsetjmp(jmp_env, 1)) {
-		pr_inf("%s: premature SIGSEGV caught, skipping stressor\n",
+		pr_inf_skip("%s: premature SIGSEGV caught, skipping stressor\n",
 			args->name);
 		return EXIT_NO_RESOURCE;
 	}
@@ -813,6 +797,8 @@ static int stress_cache(const stress_args_t *args)
 		}
 
 		if (!(cache_flags & FLAGS_CACHE_NOAFF) || !pinned) {
+			cpu_set_t mask;
+
 			CPU_ZERO(&mask);
 			CPU_SET(cpu, &mask);
 			(void)sched_setaffinity(0, sizeof(mask), &mask);
@@ -874,7 +860,7 @@ next:
 
 				shim_strlcpy(ptr, mask_flag_info[j].name, buf_len);
 				buf_len -= len;
-				ptr+= len;
+				ptr += len;
 			}
 		}
 		*ptr = '\0';
