@@ -1,5 +1,5 @@
 /*
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,15 +22,27 @@
 static const stress_help_t help[] = {
 	{ NULL,	"dekker N",		"start N workers that exercise ther Dekker algorithm" },
 	{ NULL,	"dekker-ops N",		"stop after N dekker mutex bogo operations" },
-	{ NULL,	NULL,		NULL }
+	{ NULL,	NULL,			NULL }
 };
 
 #if defined(HAVE_SHIM_MFENCE)
 
-typedef struct dekker {
+typedef struct {
 	volatile bool	wants_to_enter[2];
 	volatile int	turn;
-	volatile int check;
+	volatile int	check;
+} dekker_mutex_t;
+
+/*
+ *  dekker_t is mmap'd shared and m, p0, p1 are 64 byte cache aligned
+ *  to reduce cache contention when updating metrics on p0 and p1
+ */
+typedef struct dekker {
+	dekker_mutex_t	m;
+	char 		pad1[64 - sizeof(dekker_mutex_t)];
+	stress_metrics_t p0;
+	char		pad2[64 - sizeof(stress_metrics_t)];
+	stress_metrics_t p1;
 } dekker_t;
 
 dekker_t *dekker;
@@ -38,28 +50,32 @@ dekker_t *dekker;
 static void stress_dekker_p0(const stress_args_t *args)
 {
 	int check0, check1;
+	double t;
 
-	dekker->wants_to_enter[0] = true;
+	t = stress_time_now();
+	dekker->m.wants_to_enter[0] = true;
 	shim_mfence();
-	while (LIKELY(dekker->wants_to_enter[1])) {
-		if (dekker->turn != 0) {
-			dekker->wants_to_enter[0] = false;
+	while (LIKELY(dekker->m.wants_to_enter[1])) {
+		if (dekker->m.turn != 0) {
+			dekker->m.wants_to_enter[0] = false;
 			shim_mfence();
-			while (dekker->turn != 0) {
+			while (dekker->m.turn != 0) {
 			}
-			dekker->wants_to_enter[0] = true;
+			dekker->m.wants_to_enter[0] = true;
 			shim_mfence();
 		}
 	}
 
 	/* Critical section */
-	check0 = dekker->check;
-	dekker->check++;
-	check1 = dekker->check;
+	check0 = dekker->m.check;
+	dekker->m.check++;
+	check1 = dekker->m.check;
 
-	dekker->turn = 1;
-	dekker->wants_to_enter[0] = false;
+	dekker->m.turn = 1;
+	dekker->m.wants_to_enter[0] = false;
 	shim_mfence();
+	dekker->p0.duration += stress_time_now() - t;
+	dekker->p0.count += 1.0;
 
 	if (check0 + 1 != check1) {
 		pr_fail("%s p0: dekker mutex check failed %d vs %d\n",
@@ -70,29 +86,34 @@ static void stress_dekker_p0(const stress_args_t *args)
 static void stress_dekker_p1(const stress_args_t *args)
 {
 	int check0, check1;
+	double t;
 
-	dekker->wants_to_enter[1] = true;
+	t = stress_time_now();
+
+	dekker->m.wants_to_enter[1] = true;
 	shim_mfence();
-	while (LIKELY(dekker->wants_to_enter[0])) {
-		if (dekker->turn != 1) {
-			dekker->wants_to_enter[1] = false;
+	while (LIKELY(dekker->m.wants_to_enter[0])) {
+		if (dekker->m.turn != 1) {
+			dekker->m.wants_to_enter[1] = false;
 			shim_mfence();
-			while (dekker->turn != 1) {
+			while (dekker->m.turn != 1) {
 			}
-			dekker->wants_to_enter[1] = true;
+			dekker->m.wants_to_enter[1] = true;
 			shim_mfence();
 		}
 	}
 
 	/* Critical section */
-	check0 = dekker->check;
-	dekker->check--;
-	check1 = dekker->check;
+	check0 = dekker->m.check;
+	dekker->m.check--;
+	check1 = dekker->m.check;
 	inc_counter(args);
 
-	dekker->turn = 0;
-	dekker->wants_to_enter[1] = false;
+	dekker->m.turn = 0;
+	dekker->m.wants_to_enter[1] = false;
 	shim_mfence();
+	dekker->p1.duration += stress_time_now() - t;
+	dekker->p1.count += 1.0;
 
 	if (check0 - 1 != check1) {
 		pr_fail("%s p1: dekker mutex check failed %d vs %d\n",
@@ -108,6 +129,7 @@ static int stress_dekker(const stress_args_t *args)
 {
 	const size_t sz = STRESS_MAXIMUM(args->page_size, sizeof(*dekker));
 	pid_t pid;
+	double rate, duration, count;
 
 	dekker = (dekker_t *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
@@ -137,6 +159,11 @@ static int stress_dekker(const stress_args_t *args)
 		(void)waitpid(pid, &status, 0);
 	}
 
+	duration = dekker->p0.duration + dekker->p1.duration;
+	count = dekker->p0.count + dekker->p1.count;
+	rate = (count > 0.0) ? (duration / count) : 0.0;
+	stress_metrics_set(args, 0, "nanosecs per mutex", rate * STRESS_DBL_NANOSECOND);
+
 	(void)munmap((void *)dekker, 4096);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -155,10 +182,11 @@ stressor_info_t stress_dekker_info = {
 #else
 
 stressor_info_t stress_dekker_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU,
 	.verify = VERIFY_ALWAYS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without user space memory fencing"
 };
 
 #endif
