@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King.
+ * Copyright (C) 2021-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,18 +21,14 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"hrtimers N",	  "start N workers that exercise high resolution timers" },
-	{ NULL,	"hrtimers-ops N", "stop after N bogo high-res timer bogo operations" },
 	{ NULL, "hrtimers-adjust","adjust rate to try and maximum timer rate" },
+	{ NULL,	"hrtimers-ops N", "stop after N bogo high-res timer bogo operations" },
 	{ NULL,	NULL,		  NULL }
 };
 
 static int stress_set_hrtimers_adjust(const char *opt)
 {
-        bool hrtimers_adjust = true;
-
-        (void)opt;
-
-        return stress_set_setting("hrtimers-adjust", TYPE_ID_BOOL, &hrtimers_adjust);
+        return stress_set_setting_true("hrtimers-adjust", opt);
 }
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
@@ -59,7 +55,7 @@ static long ns_delay;
 static void stress_hrtimers_set(struct itimerspec *timer)
 {
 	if (ns_delay < 0) {
-		timer->it_value.tv_nsec = (stress_mwc64() % 50000) + 1;
+		timer->it_value.tv_nsec = stress_mwc64modn(50000) + 1;
 	} else {
 		timer->it_value.tv_nsec = ns_delay;
 	}
@@ -123,7 +119,6 @@ static int stress_hrtimer_process(const stress_args_t *args, uint64_t *counter)
 	static uint64_t last_count;
 	double previous_time, dt;
 	bool hrtimer_interrupt = false;
-	int ret;
 
 	timer_counter = counter;
 	last_count = *counter;
@@ -132,8 +127,7 @@ static int stress_hrtimer_process(const stress_args_t *args, uint64_t *counter)
 	(void)sigaddset(&mask, SIGINT);
 	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
 
-	ret = stress_set_sched(getpid(), SCHED_RR, UNDEFINED, true);
-	(void)ret;
+	VOID_RET(int, stress_set_sched(getpid(), SCHED_RR, UNDEFINED, true));
 
 	start = stress_time_now();
 	if (stress_sighandler(args->name, SIGRTMIN, stress_hrtimers_handler, NULL) < 0)
@@ -159,17 +153,19 @@ static int stress_hrtimer_process(const stress_args_t *args, uint64_t *counter)
 	previous_time = stress_time_now();
 
 	do {
+		unsigned int sl_ret;
+
 		if (ns_delay < 0) {
-			ret = sleep(1);
-			if (ret == 0)
+			sl_ret = sleep(1);
+			if (sl_ret == 0)
 				hrtimer_interrupt = true;
 		} else {
-			long ns_adjust = ns_delay >> 2;
+			const long ns_adjust = ns_delay >> 2;
 			double now;
 
 			/* The sleep will be interrupted on each hrtimer tick */
-			ret = sleep(1);
-			if (ret == 0)
+			sl_ret = sleep(1);
+			if (sl_ret == 0)
 				hrtimer_interrupt = true;
 
 			now = stress_time_now();
@@ -200,7 +196,7 @@ static int stress_hrtimer_process(const stress_args_t *args, uint64_t *counter)
 	} while (stress_hrtimers_keep_stressing());
 
 	if (!hrtimer_interrupt) {
-		pr_fail("%s: did not detect any hrtimer interrupts of a sleep\n",
+		pr_dbg("%s: failed to detect hrtimer interrupts during a 1 sec sleep\n",
 			args->name);
 	}
 
@@ -232,8 +228,9 @@ static int stress_hrtimers(const stress_args_t *args)
 	counters = (uint64_t *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
 				MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (counters == MAP_FAILED) {
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: failed to mmap %zd bytes, errno=%d (%s), "
+			"skipping stressor\n",
+			args->name, sz, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -247,7 +244,6 @@ static int stress_hrtimers(const stress_args_t *args)
 		if (pids[i] == 0) {
 			/* Child */
 
-			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
 			stress_set_oom_adjustment(args->name, true);
 			(void)sched_settings_apply(true);
@@ -272,8 +268,10 @@ static int stress_hrtimers(const stress_args_t *args)
 
 	dt = stress_time_now() - start_time;
 	if (dt > 0.0) {
+		const uint64_t c = get_counter(args);
+
 		pr_inf("%s: hrtimer signals at %.3f MHz\n",
-			args->name, ((double)get_counter(args) / dt) / 1000000.0);
+			args->name, ((double)c / dt) / 1000000.0);
 	}
 
 
@@ -286,10 +284,9 @@ reap:
 	}
 	for (i = 0; i < PROCS_MAX; i++) {
 		if (pids[i] > 0) {
-			int status, ret;
+			int status;
 
-			ret = shim_waitpid(pids[i], &status, 0);
-			(void)ret;
+			VOID_RET(int, shim_waitpid(pids[i], &status, 0));
 		}
 	}
 
@@ -307,9 +304,10 @@ stressor_info_t stress_hrtimers_info = {
 };
 #else
 stressor_info_t stress_hrtimers_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without librt or hrtimer support"
 };
 #endif
