@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,24 +25,46 @@
 #include <intel-ipsec-mb.h>
 #endif
 
+typedef struct {
+	double	ops;
+	double	duration;
+} ipsec_stats_t;
+
 static const stress_help_t help[] = {
-	{ NULL,	"ipsec-mb N",	  "start N workers exercising the IPSec MB encoding" },
-	{ NULL,	"ipsec-mb-ops N", "stop after N ipsec bogo encoding operations" },
-	{ NULL, "ipsec-mb-feature F","specify CPU feature F" },
-	{ NULL,	NULL,		  NULL }
+	{ NULL,	"ipsec-mb N",		"start N workers exercising the IPSec MB encoding" },
+	{ NULL, "ipsec-mb-feature F",	"specify CPU feature F" },
+	{ NULL,	"ipsec-mb-jobs N",	"specify number of jobs to run per round (default 1)" },
+	{ NULL,	"ipsec-mb-method M",	"specify crypto/integrity method" },
+	{ NULL,	"ipsec-mb-ops N",	"stop after N ipsec bogo encoding operations" },
+	{ NULL,	NULL,		  	NULL }
 };
 
 static int stress_set_ipsec_mb_feature(const char *opt);
+static int stress_set_ipsec_mb_method(const char *opt);
+
+/*
+ *  stress_set_ipsec_mb_jobs()
+ *      set number of jobs per round
+ */
+static int stress_set_ipsec_mb_jobs(const char *opt)
+{
+	int ipsec_mb_jobs;
+
+	ipsec_mb_jobs = (int)stress_get_int32(opt);
+	stress_check_range("ipsec-mb-jobs", (uint64_t)ipsec_mb_jobs, 1, 65536);
+	return stress_set_setting("ipsec-mb-jobs", TYPE_ID_INT, &ipsec_mb_jobs);
+}
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ OPT_ipsec_mb_feature,	stress_set_ipsec_mb_feature },
+	{ OPT_ipsec_mb_jobs,	stress_set_ipsec_mb_jobs },
+	{ OPT_ipsec_mb_method,	stress_set_ipsec_mb_method },
 	{ 0,                    NULL }
 };
 
 #if defined(HAVE_INTEL_IPSEC_MB_H) &&	\
     defined(HAVE_LIB_IPSEC_MB) &&	\
-    defined(STRESS_ARCH_X86) &&		\
-    (defined(__x86_64__) || defined(__x86_64)) && \
+    defined(STRESS_ARCH_X86_64) &&	\
     defined(IMB_FEATURE_SSE4_2) &&	\
     defined(IMB_FEATURE_CMOV) &&	\
     defined(IMB_FEATURE_AESNI) &&	\
@@ -50,77 +72,102 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
     defined(IMB_FEATURE_AVX2) &&	\
     defined(IMB_FEATURE_AVX512_SKX)
 
-#define FEATURE_SSE		(IMB_FEATURE_SSE4_2 | IMB_FEATURE_CMOV | IMB_FEATURE_AESNI)
-#define FEATURE_AVX		(IMB_FEATURE_AVX | IMB_FEATURE_CMOV | IMB_FEATURE_AESNI)
-#define FEATURE_AVX2		(FEATURE_AVX | IMB_FEATURE_AVX2)
-#define FEATURE_AVX512		(FEATURE_AVX2 | IMB_FEATURE_AVX512_SKX)
+typedef void (*ipsec_func_t)(
+        const stress_args_t *args,
+        struct MB_MGR *mb_mgr,
+        const uint8_t *data,
+        const size_t data_len,
+        const int jobs);
 
 typedef struct {
-	const uint64_t features;
-	const char *name;
-	void (*init_func)(MB_MGR *p_mgr);
-} stress_init_mb_t;
+	ipsec_func_t	func;
+	char 		*name;
+} stress_ipsec_funcs_t;
 
-static stress_init_mb_t init_mb[] = {
-	{ FEATURE_SSE,		"sse",		init_mb_mgr_sse },
-	{ FEATURE_AVX,		"avx",		init_mb_mgr_avx },
-	{ FEATURE_AVX2,		"avx2",		init_mb_mgr_avx2 },
-	{ FEATURE_AVX512,	"avx512",	init_mb_mgr_avx512 },
+typedef struct {
+	uint64_t	features;
+	void 		(*init_func)(MB_MGR *p_mgr);
+	char 		*name;
+	bool		supported;
+	ipsec_stats_t	stats;
+} stress_ipsec_features_t;
+
+static stress_ipsec_features_t mb_features[] = {
+	{
+		IMB_FEATURE_AVX | IMB_FEATURE_CMOV | IMB_FEATURE_AESNI,
+		init_mb_mgr_avx,
+		"avx",
+		false,
+		{ 0.0, 0.0 }
+	},
+	{
+		IMB_FEATURE_AVX2 | IMB_FEATURE_AVX | IMB_FEATURE_CMOV |
+		IMB_FEATURE_AESNI,
+		init_mb_mgr_avx2,
+		"avx2",
+		false,
+		{ 0.0, 0.0 }
+	},
+	{
+		IMB_FEATURE_AVX512_SKX | IMB_FEATURE_AVX2 |IMB_FEATURE_AVX |
+		IMB_FEATURE_CMOV | IMB_FEATURE_AESNI,
+		init_mb_mgr_avx512,
+		"avx512",
+		false,
+		{ 0.0, 0.0 }
+	},
+	{
+		IMB_FEATURE_SSE4_2 | IMB_FEATURE_CMOV,
+		init_mb_mgr_sse,
+		"noaesni",
+		false,
+		{ 0.0, 0.0 }
+	},
+	{
+		IMB_FEATURE_SSE4_2 | IMB_FEATURE_CMOV | IMB_FEATURE_AESNI,
+		init_mb_mgr_sse,
+		"sse",
+		false,
+		{ 0.0, 0.0 }
+	},
 };
 
 static int stress_set_ipsec_mb_feature(const char *opt)
 {
 	size_t i;
 
-	for (i = 0; i < SIZEOF_ARRAY(init_mb); i++) {
-		if (!strcmp(opt, init_mb[i].name)) {
-			uint64_t ipsec_mb_feature = init_mb[i].features;
-
-			return stress_set_setting("ipsec-mb-feature", TYPE_ID_INT, &ipsec_mb_feature);
-		}
+	for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+		if (!strcmp(opt, mb_features[i].name))
+			return stress_set_setting("ipsec-mb-feature", TYPE_ID_SIZE_T, &i);
 	}
 
 	(void)fprintf(stderr, "invalid ipsec-mb-feature '%s', allowed options are:", opt);
-	for (i = 0; i < SIZEOF_ARRAY(init_mb); i++) {
-		(void)fprintf(stderr, " %s", init_mb[i].name);
+	for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+		(void)fprintf(stderr, " %s", mb_features[i].name);
 	}
 	(void)fprintf(stderr, "\n");
 
 	return -1;
 }
 
-static const char *stress_get_ipsec_mb_feature(const uint64_t feature)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(init_mb); i++) {
-		if (init_mb[i].features == feature)
-			return init_mb[i].name;
-	}
-	return "(unknown)";
-}
-
 /*
  *  stress_ipsec_mb_features()
  *	get list of CPU feature bits
  */
 static uint64_t stress_ipsec_mb_features(const stress_args_t *args, MB_MGR *p_mgr)
 {
-	uint64_t features;
-
-	features = p_mgr->features;
+	const uint64_t features = p_mgr->features;
 
 	if (args->instance == 0) {
-		char str[128] = "";
+		char str[256] = "";
+		size_t i;
 
-		if ((features & FEATURE_SSE) == FEATURE_SSE)
-			strcat(str, " sse");
-		if ((features & FEATURE_AVX) == FEATURE_AVX)
-			strcat(str, " avx");
-		if ((features & FEATURE_AVX2) == FEATURE_AVX2)
-			strcat(str, " avx2");
-		if ((features & FEATURE_AVX512) == FEATURE_AVX512)
-			strcat(str, " avx512");
+		for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+			if ((features & mb_features[i].features) == mb_features[i].features) {
+				shim_strlcat(str, " ", sizeof(str));
+				shim_strlcat(str, mb_features[i].name, sizeof(str));
+			}
+		}
 
 		pr_inf("%s: features:%s\n", args->name, str);
 	}
@@ -167,11 +214,9 @@ static inline void stress_job_empty(struct MB_MGR *mb_mgr)
 
 static inline struct JOB_AES_HMAC *stress_job_get_next(struct MB_MGR *mb_mgr)
 {
-	struct JOB_AES_HMAC *job;
+	struct JOB_AES_HMAC *job = IMB_GET_NEXT_JOB(mb_mgr);
 
-	job = IMB_GET_NEXT_JOB(mb_mgr);
 	(void)memset(job, 0, sizeof(*job));
-
 	return job;
 }
 
@@ -185,11 +230,12 @@ static void stress_job_check_status(
 	struct JOB_AES_HMAC *job,
 	int *jobs_done)
 {
-	(*jobs_done)++;
-
 	if (job->status != STS_COMPLETED) {
 		pr_err("%s: %s: job not completed\n",
 			args->name, name);
+	} else {
+		(*jobs_done)++;
+		inc_counter(args);
 	}
 }
 
@@ -208,7 +254,25 @@ static void stress_jobs_done(
 			args->name, name, jobs_done, jobs);
 }
 
-static void stress_sha(
+static void *stress_alloc_aligned(const size_t nmemb, const size_t size, const size_t alignment)
+{
+	const size_t sz = nmemb * size;
+	void *ptr;
+
+#if defined(HAVE_POSIX_MEMALIGN)
+	if (posix_memalign(&ptr, alignment, sz) == 0)
+		return ptr;
+#elif defined(HAVE_ALIGNED_ALLOC)
+	return aligned_alloc(alignment, sz);
+#elif defined(HAVE_MEMALIGN)
+	return memalign(aiglment, sz);
+#endif
+	return NULL;
+}
+
+#define SHA_DIGEST_SIZE		(64)
+
+static void stress_ipsec_sha(
 	const stress_args_t *args,
 	struct MB_MGR *mb_mgr,
 	const uint8_t *data,
@@ -216,14 +280,17 @@ static void stress_sha(
 	const int jobs)
 {
 	int j, jobs_done = 0;
-	const int sha_digest_size = 64;
 	struct JOB_AES_HMAC *job;
 	uint8_t padding[16];
-	const size_t alloc_len = sha_digest_size + (sizeof(padding) * 2);
+	const size_t alloc_len = SHA_DIGEST_SIZE + (sizeof(padding) * 2);
 	uint8_t *auth;
-	uint8_t auth_data[alloc_len * jobs];
+	uint8_t *auth_data;
 	static const char name[] = "sha";
 
+	auth_data = (uint8_t *)stress_alloc_aligned((size_t)jobs, alloc_len, 16);
+	if (!auth_data)
+		return;
+
 	stress_job_empty(mb_mgr);
 
 	for (auth = auth_data, j = 0; j < jobs; j++, auth += alloc_len) {
@@ -231,7 +298,7 @@ static void stress_sha(
 		job->cipher_direction = ENCRYPT;
 		job->chain_order = HASH_CIPHER;
 		job->auth_tag_output = auth + sizeof(padding);
-		job->auth_tag_output_len_in_bytes = sha_digest_size;
+		job->auth_tag_output_len_in_bytes = SHA_DIGEST_SIZE;
 		job->src = data;
 		job->msg_len_to_hash_in_bytes = data_len;
 		job->cipher_mode = NULL_CIPHER;
@@ -247,9 +314,10 @@ static void stress_sha(
 
 	stress_jobs_done(args, name, jobs, jobs_done);
 	stress_job_empty(mb_mgr);
+	free(auth_data);
 }
 
-static void stress_des(
+static void stress_ipsec_des(
 	const stress_args_t *args,
 	struct MB_MGR *mb_mgr,
 	const uint8_t *data,
@@ -259,14 +327,18 @@ static void stress_des(
 	int j, jobs_done = 0;
 	struct JOB_AES_HMAC *job;
 
-	uint8_t encoded[jobs * data_len] ALIGNED(16);
+	uint8_t *encoded;
 	uint8_t k[32] ALIGNED(16);
 	uint8_t iv[16] ALIGNED(16);
-	uint32_t enc_keys[15*4] ALIGNED(16);
-	uint32_t dec_keys[15*4] ALIGNED(16);
+	uint32_t enc_keys[15 * 4] ALIGNED(16);
+	uint32_t dec_keys[15 * 4] ALIGNED(16);
 	uint8_t *dst;
 	static const char name[] = "des";
 
+	encoded = (uint8_t *)stress_alloc_aligned((size_t)jobs, data_len, 16);
+	if (!encoded)
+		return;
+
 	stress_rnd_fill(k, sizeof(k));
 	stress_rnd_fill(iv, sizeof(iv));
 	stress_job_empty(mb_mgr);
@@ -299,9 +371,10 @@ static void stress_des(
 
 	stress_jobs_done(args, name, jobs, jobs_done);
 	stress_job_empty(mb_mgr);
+	free(encoded);
 }
 
-static void stress_cmac(
+static void stress_ipsec_cmac(
 	const stress_args_t *args,
 	struct MB_MGR *mb_mgr,
 	const uint8_t *data,
@@ -315,10 +388,14 @@ static void stress_cmac(
 	uint32_t expkey[4 * 15] ALIGNED(16);
 	uint32_t dust[4 * 15] ALIGNED(16);
 	uint32_t skey1[4], skey2[4];
-	uint8_t output[jobs * data_len] ALIGNED(16);
+	uint8_t *output;
 	static const char name[] = "cmac";
 	uint8_t *dst;
 
+	output = (uint8_t *)stress_alloc_aligned((size_t)jobs, data_len, 16);
+	if (!output)
+		return;
+
 	stress_rnd_fill(key, sizeof(key));
 	IMB_AES_KEYEXP_128(mb_mgr, key, expkey, dust);
 	IMB_AES_CMAC_SUBKEY_GEN_128(mb_mgr, expkey, skey1, skey2);
@@ -349,9 +426,10 @@ static void stress_cmac(
 
 	stress_jobs_done(args, name, jobs, jobs_done);
 	stress_job_empty(mb_mgr);
+	free(output);
 }
 
-static void stress_ctr(
+static void stress_ipsec_ctr(
 	const stress_args_t *args,
 	struct MB_MGR *mb_mgr,
 	const uint8_t *data,
@@ -361,7 +439,7 @@ static void stress_ctr(
 	int j, jobs_done = 0;
 	struct JOB_AES_HMAC *job;
 
-	uint8_t encoded[jobs * data_len] ALIGNED(16);
+	uint8_t *encoded;
 	uint8_t key[32] ALIGNED(16);
 	uint8_t iv[12] ALIGNED(16);		/* 4 byte nonce + 8 byte IV */
 	uint32_t expkey[4 * 15] ALIGNED(16);
@@ -369,6 +447,10 @@ static void stress_ctr(
 	uint8_t *dst;
 	static const char name[] = "ctr";
 
+	encoded = (uint8_t *)stress_alloc_aligned((size_t)jobs, data_len, 16);
+	if (!encoded)
+		return;
+
 	stress_rnd_fill(key, sizeof(key));
 	stress_rnd_fill(iv, sizeof(iv));
 	IMB_AES_KEYEXP_256(mb_mgr, key, expkey, dust);
@@ -399,9 +481,13 @@ static void stress_ctr(
 
 	stress_jobs_done(args, name, jobs, jobs_done);
 	stress_job_empty(mb_mgr);
+	free(encoded);
 }
 
-static void stress_hmac_md5(
+#define HMAC_MD5_DIGEST_SIZE	(16)
+#define MMAC_MD5_BLOCK_SIZE	(64)
+
+static void stress_ipsec_hmac_md5(
 	const stress_args_t *args,
 	struct MB_MGR *mb_mgr,
 	const uint8_t *data,
@@ -412,16 +498,18 @@ static void stress_hmac_md5(
 	size_t i;
 	struct JOB_AES_HMAC *job;
 
-	const int digest_size = 16;
-	const int block_size = 64;
-	uint8_t key[block_size] ALIGNED(16);
-	uint8_t buf[block_size] ALIGNED(16);
-	uint8_t ipad_hash[digest_size] ALIGNED(16);
-	uint8_t opad_hash[digest_size] ALIGNED(16);
-	uint8_t output[jobs * digest_size] ALIGNED(16);
+	uint8_t key[MMAC_MD5_BLOCK_SIZE] ALIGNED(16);
+	uint8_t buf[MMAC_MD5_BLOCK_SIZE] ALIGNED(16);
+	uint8_t ipad_hash[HMAC_MD5_DIGEST_SIZE] ALIGNED(16);
+	uint8_t opad_hash[HMAC_MD5_DIGEST_SIZE] ALIGNED(16);
+	uint8_t *output;
 	uint8_t *dst;
 	static const char name[] = "hmac_md5";
 
+	output = (uint8_t *)stress_alloc_aligned((size_t)jobs, HMAC_MD5_DIGEST_SIZE, 16);
+	if (!output)
+		return;
+
 	stress_rnd_fill(key, sizeof(key));
 	for (i = 0; i < sizeof(key); i++)
 		buf[i] = key[i] ^ 0x36;
@@ -432,7 +520,7 @@ static void stress_hmac_md5(
 
 	stress_job_empty(mb_mgr);
 
-	for (dst = output, j = 0; j < jobs; j++, dst += digest_size) {
+	for (dst = output, j = 0; j < jobs; j++, dst += HMAC_MD5_DIGEST_SIZE) {
 		job = stress_job_get_next(mb_mgr);
 		job->aes_enc_key_expanded = NULL;
 		job->aes_dec_key_expanded = NULL;
@@ -441,7 +529,7 @@ static void stress_hmac_md5(
 		job->dst = NULL;
 		job->aes_key_len_in_bytes = 0;
 		job->auth_tag_output = dst;
-		job->auth_tag_output_len_in_bytes = digest_size;
+		job->auth_tag_output_len_in_bytes = HMAC_MD5_DIGEST_SIZE;
 		job->iv = NULL;
 		job->iv_len_in_bytes = 0;
 		job->src = data;
@@ -464,9 +552,13 @@ static void stress_hmac_md5(
 
 	stress_jobs_done(args, name, jobs, jobs_done);
 	stress_job_empty(mb_mgr);
+	free(output);
 }
 
-static void stress_hmac_sha1(
+#define HMAC_SHA1_DIGEST_SIZE	(20)
+#define HMAC_SHA1_BLOCK_SIZE	(64)
+
+static void stress_ipsec_hmac_sha1(
 	const stress_args_t *args,
 	struct MB_MGR *mb_mgr,
 	const uint8_t *data,
@@ -477,16 +569,18 @@ static void stress_hmac_sha1(
 	size_t i;
 	struct JOB_AES_HMAC *job;
 
-	const int digest_size = 20;
-	const int block_size = 64;
-	uint8_t key[block_size] ALIGNED(16);
-	uint8_t buf[block_size] ALIGNED(16);
-	uint8_t ipad_hash[digest_size] ALIGNED(16);
-	uint8_t opad_hash[digest_size] ALIGNED(16);
-	uint8_t output[jobs * digest_size] ALIGNED(16);
+	uint8_t key[HMAC_SHA1_BLOCK_SIZE] ALIGNED(16);
+	uint8_t buf[HMAC_SHA1_BLOCK_SIZE] ALIGNED(16);
+	uint8_t ipad_hash[HMAC_SHA1_DIGEST_SIZE] ALIGNED(16);
+	uint8_t opad_hash[HMAC_SHA1_DIGEST_SIZE] ALIGNED(16);
+	uint8_t *output;
 	uint8_t *dst;
 	static const char name[] = "hmac_sha1";
 
+	output = (uint8_t *)stress_alloc_aligned((size_t)jobs, HMAC_SHA1_DIGEST_SIZE, 16);
+	if (!output)
+		return;
+
 	stress_rnd_fill(key, sizeof(key));
 	for (i = 0; i < sizeof(key); i++)
 		buf[i] = key[i] ^ 0x36;
@@ -497,7 +591,7 @@ static void stress_hmac_sha1(
 
 	stress_job_empty(mb_mgr);
 
-	for (dst = output, j = 0; j < jobs; j++, dst += digest_size) {
+	for (dst = output, j = 0; j < jobs; j++, dst += HMAC_SHA1_DIGEST_SIZE) {
 		job = stress_job_get_next(mb_mgr);
 		job->aes_enc_key_expanded = NULL;
 		job->aes_dec_key_expanded = NULL;
@@ -506,7 +600,7 @@ static void stress_hmac_sha1(
 		job->dst = NULL;
 		job->aes_key_len_in_bytes = 0;
 		job->auth_tag_output = dst;
-		job->auth_tag_output_len_in_bytes = digest_size;
+		job->auth_tag_output_len_in_bytes = HMAC_SHA1_DIGEST_SIZE;
 		job->iv = NULL;
 		job->iv_len_in_bytes = 0;
 		job->src = data;
@@ -529,9 +623,10 @@ static void stress_hmac_sha1(
 
 	stress_jobs_done(args, name, jobs, jobs_done);
 	stress_job_empty(mb_mgr);
+	free(output);
 }
 
-static void stress_hmac_sha512(
+static void stress_ipsec_hmac_sha512(
 	const stress_args_t *args,
 	struct MB_MGR *mb_mgr,
 	const uint8_t *data,
@@ -542,17 +637,19 @@ static void stress_hmac_sha512(
 	size_t i;
 	struct JOB_AES_HMAC *job;
 
-	const int digest_size = SHA512_DIGEST_SIZE_IN_BYTES;
-	const int block_size = SHA_512_BLOCK_SIZE;
-	uint8_t rndkey[block_size] ALIGNED(16);
-	uint8_t key[block_size] ALIGNED(16);
-	uint8_t buf[block_size] ALIGNED(16);
-	uint8_t ipad_hash[digest_size] ALIGNED(16);
-	uint8_t opad_hash[digest_size] ALIGNED(16);
-	uint8_t output[jobs * digest_size] ALIGNED(16);
+	uint8_t rndkey[SHA_512_BLOCK_SIZE] ALIGNED(16);
+	uint8_t key[SHA_512_BLOCK_SIZE] ALIGNED(16);
+	uint8_t buf[SHA_512_BLOCK_SIZE] ALIGNED(16);
+	uint8_t ipad_hash[SHA512_DIGEST_SIZE_IN_BYTES] ALIGNED(16);
+	uint8_t opad_hash[SHA512_DIGEST_SIZE_IN_BYTES] ALIGNED(16);
+	uint8_t *output;
 	uint8_t *dst;
 	static const char name[] = "hmac_sha512";
 
+	output = (uint8_t *)stress_alloc_aligned((size_t)jobs, SHA512_DIGEST_SIZE_IN_BYTES, 16);
+	if (!output)
+		return;
+
 	stress_rnd_fill(rndkey, sizeof(rndkey));
 	(void)memset(key, 0, sizeof(key));
 
@@ -567,7 +664,7 @@ static void stress_hmac_sha512(
 
 	stress_job_empty(mb_mgr);
 
-	for (dst = output, j = 0; j < jobs; j++, dst += digest_size) {
+	for (dst = output, j = 0; j < jobs; j++, dst += SHA512_DIGEST_SIZE_IN_BYTES) {
 		job = stress_job_get_next(mb_mgr);
 		job->aes_enc_key_expanded = NULL;
 		job->aes_dec_key_expanded = NULL;
@@ -576,7 +673,7 @@ static void stress_hmac_sha512(
 		job->dst = NULL;
 		job->aes_key_len_in_bytes = 0;
 		job->auth_tag_output = dst;
-		job->auth_tag_output_len_in_bytes = digest_size;
+		job->auth_tag_output_len_in_bytes = SHA512_DIGEST_SIZE_IN_BYTES;
 		job->iv = NULL;
 		job->iv_len_in_bytes = 0;
 		job->src = data;
@@ -599,6 +696,87 @@ static void stress_hmac_sha512(
 
 	stress_jobs_done(args, name, jobs, jobs_done);
 	stress_job_empty(mb_mgr);
+	free(output);
+}
+
+static void stress_ipsec_all(
+	const stress_args_t *args,
+	struct MB_MGR *mb_mgr,
+	const uint8_t *data,
+	const size_t data_len,
+	const int jobs);
+
+static stress_ipsec_funcs_t stress_ipsec_funcs[] = {
+	{ stress_ipsec_all,		"all",		},
+	{ stress_ipsec_cmac,		"cmac",		},
+	{ stress_ipsec_ctr,		"ctr",		},
+	{ stress_ipsec_des,		"des",		},
+	{ stress_ipsec_hmac_md5,	"hmac-md5",	},
+	{ stress_ipsec_hmac_sha1,	"hmac-sha1",	},
+	{ stress_ipsec_hmac_sha512,	"hmac-sha512",	},
+	{ stress_ipsec_sha,		"sha",		},
+};
+
+static void stress_ipsec_call_func(
+        const stress_args_t *args,
+        struct MB_MGR *mb_mgr,
+        const uint8_t *data,
+        const size_t data_len,
+        const int jobs,
+	const size_t func_index)
+{
+	size_t i;
+
+	for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+		if (mb_features[i].supported) {
+			double t;
+			uint64_t c;
+
+			c = get_counter(args);
+			t = stress_time_now();
+
+			mb_features[i].init_func(mb_mgr);
+			stress_ipsec_funcs[func_index].func(args, mb_mgr, data, data_len, jobs);
+
+			mb_features[i].stats.duration += (stress_time_now() - t);
+			mb_features[i].stats.ops += (double)(get_counter(args) - c);
+		}
+	}
+}
+
+/*
+ *  stress_ipsec_all()
+ *	exercise all
+ */
+static void stress_ipsec_all(
+	const stress_args_t *args,
+	struct MB_MGR *mb_mgr,
+	const uint8_t *data,
+	const size_t data_len,
+	const int jobs)
+{
+	size_t i;
+
+	for (i = 1; keep_stressing(args) && (i < SIZEOF_ARRAY(stress_ipsec_funcs)); i++)
+		stress_ipsec_call_func(args, mb_mgr, data, data_len, jobs, i);
+}
+
+static int stress_set_ipsec_mb_method(const char *opt)
+{
+	size_t i;
+
+	for (i = 0; i < SIZEOF_ARRAY(stress_ipsec_funcs); i++) {
+		if (!strcmp(opt, stress_ipsec_funcs[i].name))
+			return stress_set_setting("ipsec-mb-method", TYPE_ID_SIZE_T, &i);
+	}
+
+	(void)fprintf(stderr, "invalid ipsec-mb-method '%s', allowed options are:", opt);
+	for (i = 0; i < SIZEOF_ARRAY(stress_ipsec_funcs); i++) {
+		(void)fprintf(stderr, " %s", stress_ipsec_funcs[i].name);
+	}
+	(void)fprintf(stderr, "\n");
+
+	return -1;
 }
 
 /*
@@ -607,105 +785,101 @@ static void stress_hmac_sha512(
  */
 static int stress_ipsec_mb(const stress_args_t *args)
 {
-	MB_MGR *p_mgr = NULL;
+	MB_MGR *mb_mgr = NULL;
 	uint64_t features;
 	uint8_t data[8192] ALIGNED(64);
-	const size_t n_features = SIZEOF_ARRAY(init_mb);
-	double t[n_features];
-	size_t i;
-	uint64_t count = 0;
+	size_t i, j;
 	bool got_features = false;
-	uint64_t ipsec_mb_feature = ~0ULL;
+	size_t ipsec_mb_feature = 0;
+	size_t ipsec_mb_method = 0;
+	int ipsec_mb_jobs = 128;
+
+	(void)stress_get_setting("ipsec-mb-jobs", &ipsec_mb_jobs);
+	(void)stress_get_setting("ipsec-mb-method", &ipsec_mb_method);
 
-	p_mgr = alloc_mb_mgr(0);
-	if (!p_mgr) {
-		if (args->instance == 0)
-			pr_inf_skip("%s: failed to setup Intel IPSec MB library, skipping\n", args->name);
-		return EXIT_NO_RESOURCE;
-	}
 	if (imb_get_version() < IMB_VERSION(0, 51, 0)) {
 		if (args->instance == 0)
 			pr_inf_skip("%s: version %s of Intel IPSec MB library is too low, skipping\n",
 				args->name, imb_get_version_str());
-		free_mb_mgr(p_mgr);
 		return EXIT_NOT_IMPLEMENTED;
 	}
 
-	features = stress_ipsec_mb_features(args, p_mgr);
-	for (i = 0; i < n_features; i++) {
-		t[i] = 0.0;
+	mb_mgr = alloc_mb_mgr(0);
+	if (!mb_mgr) {
+		if (args->instance == 0)
+			pr_inf_skip("%s: failed to setup Intel IPSec MB library, skipping\n", args->name);
+		return EXIT_NO_RESOURCE;
 	}
 
-	for (i = 0; i < n_features; i++) {
-		if ((init_mb[i].features & features) == init_mb[i].features) {
-			got_features = true;
-			break;
-		}
+	features = stress_ipsec_mb_features(args, mb_mgr);
+
+	for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+		mb_features[i].supported =
+				((mb_features[i].features & features) == mb_features[i].features);
+		got_features |= mb_features[i].supported;
+		mb_features[i].stats.ops = 0.0;
+		mb_features[i].stats.duration = 0.0;
 	}
 	if (!got_features) {
 		if (args->instance == 0)
 			pr_inf_skip("%s: not enough CPU features to support Intel IPSec MB library, skipping\n", args->name);
-		free_mb_mgr(p_mgr);
+		free_mb_mgr(mb_mgr);
 		return EXIT_NOT_IMPLEMENTED;
 	}
 
 	if (stress_get_setting("ipsec-mb-feature", &ipsec_mb_feature)) {
-		const char *feature_name = stress_get_ipsec_mb_feature(ipsec_mb_feature);
+		const char *feature_name = mb_features[ipsec_mb_feature].name;
+
+		if (!mb_features[ipsec_mb_feature].supported) {
+			if (args->instance == 0) {
 
-		if ((ipsec_mb_feature & features) != ipsec_mb_feature) {
-			if (args->instance == 0)
 				pr_inf_skip("%s: requested ipsec-mb-feature feature '%s' is not supported, skipping\n",
 					args->name, feature_name);
-			free_mb_mgr(p_mgr);
+			}
+			free_mb_mgr(mb_mgr);
 			return EXIT_NOT_IMPLEMENTED;
 		}
-		features = ipsec_mb_feature;
+		for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+			mb_features[i].supported = (i == ipsec_mb_feature);
+		}
 		if (args->instance == 0)
 			pr_inf("%s: using just feature '%s'\n", args->name, feature_name);
 	}
 
 	stress_rnd_fill(data, sizeof(data));
-
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		for (i = 0; i < n_features; i++) {
-			if ((init_mb[i].features & features) == init_mb[i].features) {
-				double t1, t2;
-
-				t1 = stress_time_now();
-				init_mb[i].init_func(p_mgr);
-				stress_cmac(args, p_mgr, data, sizeof(data), 1);
-				stress_ctr(args, p_mgr, data, sizeof(data), 1);
-				stress_des(args, p_mgr, data, sizeof(data), 1);
-				stress_hmac_md5(args, p_mgr, data, sizeof(data), 1);
-				stress_hmac_sha1(args, p_mgr, data, sizeof(data), 1);
-				stress_hmac_sha512(args, p_mgr, data, sizeof(data), 1);
-				stress_sha(args, p_mgr, data, sizeof(data), 1);
-				t2 = stress_time_now();
-				t[i] += (t2 - t1);
+		for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+			if (mb_features[i].supported) {
+				mb_features[i].init_func(mb_mgr);
+				stress_ipsec_call_func(args, mb_mgr, data, sizeof(data), ipsec_mb_jobs, ipsec_mb_method);
 			}
 		}
-		count++;
-		inc_counter(args);
 	} while (keep_stressing(args));
 
-	for (i = 0; i < n_features; i++) {
-		if (((init_mb[i].features & features) == init_mb[i].features) && (t[i] > 0.0)) {
+	pr_lock();
+	for (i = 0, j = 0; i < SIZEOF_ARRAY(mb_features); i++) {
+		const ipsec_stats_t *stats = &mb_features[i].stats;
+
+		if (stats->duration > 0.0) {
 			char tmp[32];
+			const double rate = stats->ops / stats->duration;
+			const char *name = mb_features[i].name;
 
-			pr_dbg("%s: %s %.3f bogo/ops per second\n",
-				args->name, init_mb[i].name,
-				(double)count / t[i]);
+			pr_dbg("%s: %s %.3f bogo ops per sec\n",
+				args->name, name, rate);
 
-			(void)snprintf(tmp, sizeof(tmp), "%s bogo/ops per second", init_mb[i].name);
-			stress_misc_stats_set(args->misc_stats, i, tmp, (double)count / t[i]);
+			(void)snprintf(tmp, sizeof(tmp), "%s bogo ops per sec", name);
+			stress_metrics_set(args, j, tmp, rate);
+			j++;
 		}
 	}
+	pr_unlock();
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	free_mb_mgr(p_mgr);
+	free_mb_mgr(mb_mgr);
 
 	return EXIT_SUCCESS;
 }
@@ -719,6 +893,14 @@ stressor_info_t stress_ipsec_mb_info = {
 };
 #else
 
+static int stress_set_ipsec_mb_method(const char *opt)
+{
+	(void)opt;
+
+	pr_inf("option --ipsec-mb-method not supported on this system.\n");
+	return -1;
+}
+
 static int stress_set_ipsec_mb_feature(const char *opt)
 {
 	(void)opt;
@@ -736,10 +918,11 @@ static int stress_ipsec_mb_supported(const char *name)
 }
 
 stressor_info_t stress_ipsec_mb_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.supported = stress_ipsec_mb_supported,
 	.opt_set_funcs = opt_set_funcs,
 	.class = CLASS_CPU,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built on non-x86-64 without IPSec MB library"
 };
 #endif
