@@ -32,7 +32,8 @@ struct list_entry;
 
 typedef void (*stress_list_func)(const stress_args_t *args,
 				 const size_t n,
-				 struct list_entry *data);
+				 struct list_entry *data,
+				 stress_metrics_t *metrics);
 
 typedef struct {
 	const char              *name;  /* human readable form of stressor */
@@ -49,8 +50,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		 NULL }
 };
 
-#if defined(HAVE_SYS_QUEUE_H)
-
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
 
@@ -64,8 +63,10 @@ static sigjmp_buf jmp_env;
     defined(CIRCLEQ_INSERT_TAIL) &&	\
     defined(CIRCLEQ_FOREACH) &&		\
     defined(CIRCLEQ_FIRST) &&		\
-    defined(CIRCLEQ_REMOVE)
+    defined(CIRCLEQ_REMOVE) &&		\
+    defined(HAVE_SYS_QUEUE_H)
 #define HAVE_SYS_QUEUE_CIRCLEQ
+CIRCLEQ_HEAD(circleqhead, list_entry);
 #endif
 
 #if defined(LIST_ENTRY) &&		\
@@ -75,8 +76,10 @@ static sigjmp_buf jmp_env;
     defined(LIST_FOREACH) &&		\
     defined(LIST_EMPTY) &&		\
     defined(LIST_FIRST) &&		\
-    defined(LIST_REMOVE)
+    defined(LIST_REMOVE) &&		\
+    defined(HAVE_SYS_QUEUE_H)
 #define HAVE_SYS_QUEUE_LIST
+LIST_HEAD(listhead, list_entry);
 #endif
 
 #if defined(SLIST_ENTRY) &&		\
@@ -85,8 +88,10 @@ static sigjmp_buf jmp_env;
     defined(SLIST_INSERT_HEAD) &&	\
     defined(SLIST_FOREACH) &&		\
     defined(SLIST_EMPTY) &&		\
-    defined(SLIST_REMOVE_HEAD)
+    defined(SLIST_REMOVE_HEAD) &&	\
+    defined(HAVE_SYS_QUEUE_H)
 #define HAVE_SYS_QUEUE_SLIST
+SLIST_HEAD(slisthead, list_entry);
 #endif
 
 #if defined(STAILQ_ENTRY) &&		\
@@ -95,8 +100,10 @@ static sigjmp_buf jmp_env;
     defined(STAILQ_INSERT_TAIL) &&	\
     defined(STAILQ_FOREACH) &&		\
     defined(STAILQ_FIRST) &&		\
-    defined(STAILQ_REMOVE)
+    defined(STAILQ_REMOVE) &&		\
+    defined(HAVE_SYS_QUEUE_H)
 #define HAVE_SYS_QUEUE_STAILQ
+STAILQ_HEAD(stailhead, list_entry);
 #endif
 
 #if defined(TAILQ_ENTRY) &&		\
@@ -105,8 +112,10 @@ static sigjmp_buf jmp_env;
     defined(TAILQ_INSERT_TAIL) &&	\
     defined(TAILQ_FOREACH) &&		\
     defined(TAILQ_FIRST) &&		\
-    defined(TAILQ_REMOVE)
+    defined(TAILQ_REMOVE) &&		\
+    defined(HAVE_SYS_QUEUE_H)
 #define HAVE_SYS_QUEUE_TAILQ
+TAILQ_HEAD(tailhead, list_entry);
 #endif
 
 struct list_entry {
@@ -131,24 +140,6 @@ struct list_entry {
 	} u;
 };
 
-#if defined(HAVE_SYS_QUEUE_CIRCLEQ)
-CIRCLEQ_HEAD(circleqhead, list_entry);
-#endif
-#if defined(HAVE_SYS_QUEUE_LIST)
-LIST_HEAD(listhead, list_entry);
-#endif
-#if defined(HAVE_SYS_QUEUE_SLIST)
-SLIST_HEAD(slisthead, list_entry);
-#endif
-#if defined(HAVE_SYS_QUEUE_STAILQ)
-STAILQ_HEAD(stailhead, list_entry);
-#endif
-#if defined(HAVE_SYS_QUEUE_TAILQ)
-TAILQ_HEAD(tailhead, list_entry);
-#endif
-
-#endif
-
 /*
  *  stress_set_list_size()
  *	set list size
@@ -163,8 +154,6 @@ static int stress_set_list_size(const char *opt)
 	return stress_set_setting("list-size", TYPE_ID_UINT64, &list_size);
 }
 
-#if defined(HAVE_SYS_QUEUE_H)
-
 /*
  *  stress_list_handler()
  *	SIGALRM generic handler
@@ -182,11 +171,13 @@ static void MLOCKED_TEXT stress_list_handler(int signum)
 static void OPTIMIZE3 stress_list_slistt(
 	const stress_args_t *args,
 	const size_t n,
-	struct list_entry *data)
+	struct list_entry *data,
+	stress_metrics_t *metrics)
 {
 	size_t i;
 	register struct list_entry *entry, *head, *tail;
 	bool found = false;
+	double t;
 
 	entry = data;
 	head = entry;
@@ -197,8 +188,9 @@ static void OPTIMIZE3 stress_list_slistt(
 		tail = entry;
 	}
 
+	t = stress_time_now();
 	for (entry = head, i = 0; i < n; i++, entry++) {
-		struct list_entry *find;
+		register struct list_entry *find;
 
 		for (find = head; find; find = find->u.next) {
 			if (find == entry) {
@@ -211,6 +203,9 @@ static void OPTIMIZE3 stress_list_slistt(
 			pr_fail("%s: slistt entry #%zd not found\n",
 				args->name, i);
 	}
+	metrics->duration += stress_time_now() - t;
+	metrics->count += (double)i;
+
 	while (head) {
 		register struct list_entry *next = head->u.next;
 
@@ -223,12 +218,14 @@ static void OPTIMIZE3 stress_list_slistt(
 static void stress_list_list(
 	const stress_args_t *args,
 	const size_t n,
-	struct list_entry *data)
+	struct list_entry *data,
+	stress_metrics_t *metrics)
 {
 	size_t i;
 	struct list_entry *entry;
 	struct listhead head;
 	bool found = false;
+	double t;
 
 	LIST_INIT(&head);
 
@@ -236,6 +233,7 @@ static void stress_list_list(
 		LIST_INSERT_HEAD(&head, entry, u.list_entries);
 	}
 
+	t = stress_time_now();
 	for (entry = data, i = 0; i < n; i++, entry++) {
 		struct list_entry *find;
 
@@ -250,6 +248,9 @@ static void stress_list_list(
 			pr_fail("%s: list entry #%zd not found\n",
 				args->name, i);
 	}
+	metrics->duration += stress_time_now() - t;
+	metrics->count += (double)i;
+
 	while (!LIST_EMPTY(&head)) {
 		entry = LIST_FIRST(&head);
 		LIST_REMOVE(entry, u.list_entries);
@@ -262,12 +263,14 @@ static void stress_list_list(
 static void stress_list_slist(
 	const stress_args_t *args,
 	const size_t n,
-	struct list_entry *data)
+	struct list_entry *data,
+	stress_metrics_t *metrics)
 {
 	size_t i;
 	struct list_entry *entry;
 	struct slisthead head;
 	bool found = false;
+	double t;
 
 	SLIST_INIT(&head);
 
@@ -275,6 +278,7 @@ static void stress_list_slist(
 		SLIST_INSERT_HEAD(&head, entry, u.slist_entries);
 	}
 
+	t = stress_time_now();
 	for (entry = data, i = 0; i < n; i++, entry++) {
 		struct list_entry *find;
 
@@ -289,6 +293,9 @@ static void stress_list_slist(
 			pr_fail("%s: slist entry #%zd not found\n",
 				args->name, i);
 	}
+	metrics->duration += stress_time_now() - t;
+	metrics->count += (double)i;
+
 	while (!SLIST_EMPTY(&head)) {
 		SLIST_REMOVE_HEAD(&head, u.slist_entries);
 	}
@@ -300,12 +307,14 @@ static void stress_list_slist(
 static void stress_list_circleq(
 	const stress_args_t *args,
 	const size_t n,
-	struct list_entry *data)
+	struct list_entry *data,
+	stress_metrics_t *metrics)
 {
 	size_t i;
 	struct list_entry *entry;
 	struct circleqhead head;
 	bool found = false;
+	double t;
 
 	CIRCLEQ_INIT(&head);
 
@@ -313,6 +322,7 @@ static void stress_list_circleq(
 		CIRCLEQ_INSERT_TAIL(&head, entry, u.circleq_entries);
 	}
 
+	t = stress_time_now();
 	for (entry = data, i = 0; i < n; i++, entry++) {
 		struct list_entry *find;
 
@@ -327,6 +337,9 @@ static void stress_list_circleq(
 			pr_fail("%s: circleq entry #%zd not found\n",
 				args->name, i);
 	}
+	metrics->duration += stress_time_now() - t;
+	metrics->count += (double)i;
+
 	while ((entry = CIRCLEQ_FIRST(&head)) != (struct list_entry *)&head) {
 		CIRCLEQ_REMOVE(&head, entry, u.circleq_entries);
 	}
@@ -338,12 +351,14 @@ static void stress_list_circleq(
 static void stress_list_stailq(
 	const stress_args_t *args,
 	const size_t n,
-	struct list_entry *data)
+	struct list_entry *data,
+	stress_metrics_t *metrics)
 {
 	size_t i;
 	struct list_entry *entry;
 	struct stailhead head;
 	bool found = false;
+	double t;
 
 	STAILQ_INIT(&head);
 
@@ -351,6 +366,7 @@ static void stress_list_stailq(
 		STAILQ_INSERT_TAIL(&head, entry, u.stailq_entries);
 	}
 
+	t = stress_time_now();
 	for (entry = data, i = 0; i < n; i++, entry++) {
 		struct list_entry *find;
 
@@ -365,6 +381,9 @@ static void stress_list_stailq(
 			pr_fail("%s: stailq entry #%zd not found\n",
 				args->name, i);
 	}
+	metrics->duration += stress_time_now() - t;
+	metrics->count += (double)i;
+
 	while ((entry = STAILQ_FIRST(&head)) != NULL) {
 		STAILQ_REMOVE(&head, entry, list_entry, u.stailq_entries);
 	}
@@ -376,12 +395,14 @@ static void stress_list_stailq(
 static void stress_list_tailq(
 	const stress_args_t *args,
 	const size_t n,
-	struct list_entry *data)
+	struct list_entry *data,
+	stress_metrics_t *metrics)
 {
 	size_t i;
 	struct list_entry *entry;
 	struct tailhead head;
 	bool found = false;
+	double t;
 
 	TAILQ_INIT(&head);
 
@@ -389,6 +410,7 @@ static void stress_list_tailq(
 		TAILQ_INSERT_TAIL(&head, entry, u.tailq_entries);
 	}
 
+	t = stress_time_now();
 	for (entry = data, i = 0; i < n; i++, entry++) {
 		struct list_entry *find;
 
@@ -403,6 +425,9 @@ static void stress_list_tailq(
 			pr_fail("%s: tailq entry #%zd not found\n",
 				args->name, i);
 	}
+	metrics->duration += stress_time_now() - t;
+	metrics->count += (double)i;
+
 	while ((entry = TAILQ_FIRST(&head)) != NULL) {
 		TAILQ_REMOVE(&head, entry, u.tailq_entries);
 	}
@@ -413,35 +438,14 @@ static void stress_list_tailq(
 static void stress_list_all(
 	const stress_args_t *args,
 	const size_t n,
-	struct list_entry *data)
-{
-	(void)args;
-	(void)n;
-	(void)data;
-#if defined(HAVE_SYS_QUEUE_CIRCLEQ)
-	stress_list_circleq(args, n, data);
-#endif
-#if defined(HAVE_SYS_QUEUE_LIST)
-	stress_list_list(args, n, data);
-#endif
-#if defined(HAVE_SYS_QUEUE_SLIST)
-	stress_list_slist(args, n, data);
-#endif
-	stress_list_slistt(args, n, data);
-#if defined(HAVE_SYS_QUEUE_STAILQ)
-	stress_list_stailq(args, n, data);
-#endif
-#if defined(HAVE_SYS_QUEUE_TAILQ)
-	stress_list_tailq(args, n, data);
-#endif
-}
-#endif
+	struct list_entry *data,
+	stress_metrics_t *metrics);
+
 
 /*
  * Table of list stress methods
  */
 static const stress_list_method_info_t list_methods[] = {
-#if defined(HAVE_SYS_QUEUE_H)
 	{ "all",	stress_list_all },
 #if defined(HAVE_SYS_QUEUE_CIRCLEQ)
 	{ "circleq",	stress_list_circleq },
@@ -459,28 +463,40 @@ static const stress_list_method_info_t list_methods[] = {
 #if defined(HAVE_SYS_QUEUE_TAILQ)
 	{ "tailq",	stress_list_tailq },
 #endif
-#endif
-	{ NULL,		NULL },
 };
 
+static void stress_list_all(
+	const stress_args_t *args,
+	const size_t n,
+	struct list_entry *data,
+	stress_metrics_t *metrics)
+{
+	static size_t index = 1;
+
+	list_methods[index].func(args, n, data, &metrics[index]);
+	index++;
+	if (index >= SIZEOF_ARRAY(list_methods))
+		index = 1;
+}
+
 /*
  *  stress_set_list_method()
  *	set the default funccal stress method
  */
 static int stress_set_list_method(const char *name)
 {
-	stress_list_method_info_t const *info;
+	size_t i;
 
-	for (info = list_methods; info->func; info++) {
-		if (!strcmp(info->name, name)) {
-			stress_set_setting("list-method", TYPE_ID_UINTPTR_T, &info);
+	for (i = 0; i < SIZEOF_ARRAY(list_methods); i++) {
+		if (!strcmp(list_methods[i].name, name)) {
+			stress_set_setting("list-method", TYPE_ID_SIZE_T, &i);
 			return 0;
 		}
 	}
 
 	(void)fprintf(stderr, "list-method must be one of:");
-	for (info = list_methods; info->func; info++) {
-		(void)fprintf(stderr, " %s", info->name);
+	for (i = 0; i < SIZEOF_ARRAY(list_methods); i++) {
+		(void)fprintf(stderr, " %s", list_methods[i].name);
 	}
 	(void)fprintf(stderr, "\n");
 
@@ -493,7 +509,6 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ 0,			NULL }
 };
 
-#if defined(HAVE_SYS_QUEUE_H)
 /*
  *  stress_list()
  *	stress list
@@ -502,12 +517,20 @@ static int stress_list(const stress_args_t *args)
 {
 	uint64_t v, list_size = DEFAULT_LIST_SIZE;
 	struct list_entry *entries, *entry;
-	size_t n, i, bit;
+	size_t n, i, j, bit, list_method = 0;
 	struct sigaction old_action;
 	int ret;
-	stress_list_method_info_t const *info = &list_methods[0];
+	stress_metrics_t *metrics, list_metrics[SIZEOF_ARRAY(list_methods)];
+	stress_list_func func;
 
-	(void)stress_get_setting("list-method", &info);
+	for (i = 0; i < SIZEOF_ARRAY(list_metrics); i++) {
+		list_metrics[i].duration = 0.0;
+		list_metrics[i].count = 0.0;
+	}
+
+	(void)stress_get_setting("list-method", &list_method);
+	func = list_methods[list_method].func;
+	metrics = &list_metrics[list_method];
 
 	if (!stress_get_setting("list-size", &list_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -556,7 +579,7 @@ static int stress_list(const stress_args_t *args)
 	do {
 		uint64_t rnd;
 
-		info->func(args, n, entries);
+		func(args, n, entries, metrics);
 
 		rnd = stress_mwc64();
 		for (entry = entries, i = 0; i < n; i++, entry++) {
@@ -571,6 +594,17 @@ static int stress_list(const stress_args_t *args)
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 tidy:
+	for (i = 0, j = 0; i < SIZEOF_ARRAY(list_metrics); i++) {
+		if ((list_metrics[i].duration > 0.0) && (list_metrics[i].count > 0.0)) {
+			char msg[64];
+			const double rate = list_metrics[i].count / list_metrics[i].duration;
+
+			(void)snprintf(msg, sizeof(msg), "%s searches per second", list_methods[i].name);
+			stress_metrics_set(args, j, msg, rate);
+			j++;
+		}
+	}
+
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	free(entries);
 
@@ -584,11 +618,3 @@ stressor_info_t stress_list_info = {
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
-#else
-stressor_info_t stress_list_info = {
-	.stressor = stress_unimplemented,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
-	.help = help
-};
-#endif
