@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021 Colin Ian King
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,9 +21,20 @@
 #include "core-arch.h"
 
 static const stress_help_t help[] = {
-	{ NULL,	"lockbus N",	 "start N workers locking a memory increment" },
-	{ NULL,	"lockbus-ops N", "stop after N lockbus bogo operations" },
-	{ NULL, NULL,		 NULL }
+	{ NULL,	"lockbus N",	 	"start N workers locking a memory increment" },
+	{ NULL, "lockbus-nosplit",	"disable split locks" },
+	{ NULL,	"lockbus-ops N", 	"stop after N lockbus bogo operations" },
+	{ NULL, NULL,			NULL }
+};
+
+static int stress_set_lockbus_nosplit(const char *opt)
+{
+	return stress_set_setting_true("lockbus-nosplit", opt);
+}
+
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_lockbus_nosplit,	stress_set_lockbus_nosplit },
+	{ 0,			NULL },
 };
 
 #if (((defined(__GNUC__) || defined(__clang__)) && 	\
@@ -121,6 +132,9 @@ static int stress_lockbus(const stress_args_t *args)
 	int flags = MAP_ANONYMOUS | MAP_SHARED;
 #if defined(STRESS_ARCH_X86)
 	uint32_t *splitlock_ptr1, *splitlock_ptr2;
+	bool lockbus_nosplit = false;
+
+	(void)stress_get_setting("lockbus-nosplit", &lockbus_nosplit);
 
 	if (stress_sighandler(args->name, SIGBUS, stress_sigbus_handler, NULL) < 0)
 		return EXIT_FAILURE;
@@ -131,7 +145,7 @@ static int stress_lockbus(const stress_args_t *args)
 #endif
 	buffer = (uint32_t*)mmap(NULL, BUFFER_SIZE, PROT_READ | PROT_WRITE, flags, -1, 0);
 	if (buffer == MAP_FAILED) {
-		int rc = exit_status(errno);
+		int rc = stress_exit_status(errno);
 
 		pr_err("%s: mmap failed\n", args->name);
 		return rc;
@@ -139,17 +153,20 @@ static int stress_lockbus(const stress_args_t *args)
 
 #if defined(STRESS_ARCH_X86)
 	/* Split lock on a page boundary */
-	splitlock_ptr1 = (uint32_t *)(((uint8_t *)buffer) + args->page_size - (sizeof(*splitlock_ptr1) >> 1));
+	splitlock_ptr1 = (uint32_t *)(uintptr_t)(((uint8_t *)buffer) + args->page_size - (sizeof(*splitlock_ptr1) >> 1));
 	/* Split lock on a cache boundary */
-	splitlock_ptr2 = (uint32_t *)(((uint8_t *)buffer) + 64 - (sizeof(*splitlock_ptr2) >> 1));
-	do_splitlock = true;
+	splitlock_ptr2 = (uint32_t *)(uintptr_t)(((uint8_t *)buffer) + 64 - (sizeof(*splitlock_ptr2) >> 1));
+	do_splitlock = !lockbus_nosplit;
+	if (args->instance == 0)
+		pr_dbg("%s: splitlocks %s\n", args->name,
+			do_splitlock ? "enabled" : "disabled");
 	if (sigsetjmp(jmp_env, 1) && !keep_stressing(args))
 		goto done;
 #endif
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		uint32_t *ptr0 = buffer + ((stress_mwc32() % (BUFFER_SIZE - CHUNK_SIZE)) >> 2);
+		uint32_t *ptr0 = buffer + (stress_mwc32modn(BUFFER_SIZE - CHUNK_SIZE) >> 2);
 #if defined(STRESS_ARCH_X86)
 		uint32_t *ptr1 = do_splitlock ? splitlock_ptr1 : ptr0;
 		uint32_t *ptr2 = do_splitlock ? splitlock_ptr2 : ptr0;
@@ -207,12 +224,15 @@ done:
 stressor_info_t stress_lockbus_info = {
 	.stressor = stress_lockbus,
 	.class = CLASS_CPU_CACHE | CLASS_MEMORY,
+	.opt_set_funcs = opt_set_funcs,
 	.help = help
 };
 #else
 stressor_info_t stress_lockbus_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU_CACHE | CLASS_MEMORY,
-	.help = help
+	.opt_set_funcs = opt_set_funcs,
+	.help = help,
+	.unimplemented_reason = "built without gcc __atomic* lock builtins"
 };
 #endif
