@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,13 +18,15 @@
  *
  */
 #include "stress-ng.h"
+#include "core-sort.h"
 
 #if defined(HAVE_SEARCH_H)
 #include <search.h>
 #endif
 
+#define LSEARCH_SIZE_SHIFT	(20)
 #define MIN_LSEARCH_SIZE	(1 * KB)
-#define MAX_LSEARCH_SIZE	(1 * MB)
+#define MAX_LSEARCH_SIZE	(1U << LSEARCH_SIZE_SHIFT)	/* 1 MB */
 #define DEFAULT_LSEARCH_SIZE	(8 * KB)
 
 static const stress_help_t help[] = {
@@ -55,15 +57,6 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 
 #if defined(HAVE_LSEARCH)
 
-/*
- *  cmp()
- *	lsearch int32 comparison for sorting
- */
-static int cmp(const void *p1, const void *p2)
-{
-	return (int)(*(const int32_t *)p1 - *(const int32_t *)p2);
-}
-
 /*
  *  stress_lsearch()
  *	stress lsearch
@@ -73,6 +66,7 @@ static int stress_lsearch(const stress_args_t *args)
 	int32_t *data, *root;
 	size_t i, max;
 	uint64_t lsearch_size = DEFAULT_LSEARCH_SIZE;
+	double rate, duration = 0.0, count = 0.0, sorted = 0.0;
 
 	if (!stress_get_setting("lsearch-size", &lsearch_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -83,30 +77,38 @@ static int stress_lsearch(const stress_args_t *args)
 	max = (size_t)lsearch_size;
 
 	if ((data = calloc(max, sizeof(*data))) == NULL) {
-		pr_fail("%s: malloc failed, out of memory\n", args->name);
+		pr_inf_skip("%s: malloc failed allocating %zd integers, "
+			"out of memory, skipping stressor\n", args->name, max);
 		return EXIT_NO_RESOURCE;
 	}
 	if ((root = calloc(max, sizeof(*data))) == NULL) {
 		free(data);
-		pr_fail("%s: malloc failed, out of memory\n", args->name);
+		pr_inf_skip("%s: malloc failed allocating %zd integers , "
+			"out of memory, skipping stressor\n", args->name, max);
 		return EXIT_NO_RESOURCE;
 	}
 
+	stress_sort_data_int32_init(data, max);
+
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
+		double t;
 		size_t n = 0;
 
+		stress_sort_data_int32_shuffle(data, max);
+
 		/* Step #1, populate with data */
 		for (i = 0; keep_stressing_flag() && i < max; i++) {
-			data[i] = (int32_t)(((stress_mwc32() & 0xfff) << 20) ^ i);
-			VOID_RET(void *, lsearch(&data[i], root, &n, sizeof(*data), cmp));
+			VOID_RET(void *, lsearch(&data[i], root, &n, sizeof(*data), stress_sort_cmp_int32));
 		}
 		/* Step #2, find */
+		stress_sort_compare_reset();
+		t = stress_time_now();
 		for (i = 0; keep_stressing_flag() && i < n; i++) {
 			int32_t *result;
 
-			result = lfind(&data[i], root, &n, sizeof(*data), cmp);
+			result = lfind(&data[i], root, &n, sizeof(*data), stress_sort_cmp_int32);
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				if (result == NULL)
 					pr_fail("%s: element %zu could not be found\n", args->name, i);
@@ -115,11 +117,18 @@ static int stress_lsearch(const stress_args_t *args)
 					args->name, i, *result, data[i]);
 			}
 		}
+		duration += stress_time_now() - t;
+		count += (double)stress_sort_compare_get();
+		sorted += (double)i;
 		inc_counter(args);
 	} while (keep_stressing(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	rate = (duration > 0.0) ? count / duration : 0.0;
+	stress_metrics_set(args, 0, "lsearch comparisons per sec", rate);
+	stress_metrics_set(args, 1, "lsearch comparisons per item", count / sorted);
+
 	free(root);
 	free(data);
 	return EXIT_SUCCESS;
@@ -136,11 +145,12 @@ stressor_info_t stress_lsearch_info = {
 #else
 
 stressor_info_t stress_lsearch_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
 	.verify = VERIFY_OPTIONAL,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without libc lsearch() support"
 };
 
 #endif
