@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King.
+ * Copyright (C) 2021-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,8 +32,8 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"memrate N",		"start N workers exercised memory read/writes" },
-	{ NULL,	"memrate-ops N",	"stop after N memrate bogo operations" },
 	{ NULL,	"memrate-bytes N",	"size of memory buffer being exercised" },
+	{ NULL,	"memrate-ops N",	"stop after N memrate bogo operations" },
 	{ NULL,	"memrate-rd-mbs N",	"read rate from buffer in megabytes per second" },
 	{ NULL,	"memrate-wr-mbs N",	"write rate to buffer in megabytes per second" },
 	{ NULL,	NULL,			NULL }
@@ -100,8 +100,6 @@ static int stress_set_memrate_wr_mbs(const char *opt)
 	return stress_set_setting("memrate-wr-mbs", TYPE_ID_UINT64, &memrate_wr_mbs);
 }
 
-#define SINGLE_ARG(...) __VA_ARGS__
-
 #define STRESS_MEMRATE_READ(size, type, prefetch)		\
 static uint64_t TARGET_CLONES stress_memrate_read##size(	\
 	const stress_memrate_context_t *context,		\
@@ -237,7 +235,7 @@ static uint64_t TARGET_CLONES stress_memrate_read_rate##size(	\
 	return ((uintptr_t)ptr - (uintptr_t)start) / KB;	\
 }
 
-#define no_prefetch(ptr, arg1, arg2)	
+#define no_prefetch(ptr, arg1, arg2)
 
 #if defined(HAVE_VECMATH)
 STRESS_MEMRATE_READ(1024, stress_vint8w1024_t, no_prefetch)
@@ -311,6 +309,198 @@ static uint64_t TARGET_CLONES stress_memrate_write##size(	\
 	return ((uintptr_t)ptr - (uintptr_t)start) / KB;	\
 }
 
+#if defined(HAVE_ASM_X86_REP_STOSQ) || 	\
+    defined(HAVE_ASM_X86_REP_STOSD) ||	\
+    defined(HAVE_ASM_X86_REP_STOSW) ||	\
+    defined(HAVE_ASM_X86_REP_STOSB)
+static inline uint64_t OPTIMIZE3 stress_memrate_stos(
+	const stress_memrate_context_t *context,
+	bool *valid,
+	void (*func)(void *ptr, const uint32_t mb_loops),
+	const size_t bytes)
+{
+	uint8_t *start ALIGNED(1024) = (uint8_t *)context->start;
+	uint8_t *end ALIGNED(1024) = (uint8_t *)context->end;
+	register uint8_t *ptr;
+	const uint64_t mb_loops = MB / bytes;
+
+	for (ptr = start; keep_stressing_flag() && (ptr < end); ptr += MB) {
+		func((void *)ptr, mb_loops);
+	}
+	*valid = true;
+	return ((uintptr_t)ptr - (uintptr_t)start) / KB;
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_stos_rate(
+	const stress_memrate_context_t *context,
+	bool *valid,
+	void (*func)(void *ptr, const uint32_t mb_loops),
+	const size_t bytes)
+{
+	double t1;
+	const double dur = 1.0 / (double)context->memrate_wr_mbs;
+	double total_dur = 0.0;
+	uint8_t *start ALIGNED(1024) = (uint8_t *)context->start;
+	uint8_t *end ALIGNED(1024) = (uint8_t *)context->end;
+	register uint8_t *ptr;
+	const uint64_t mb_loops = MB / bytes;
+
+	t1 = stress_time_now();
+	for (ptr = start; keep_stressing_flag() && (ptr < end); ptr += MB) {
+		double t2, dur_remainder;
+
+		func((void *)ptr, mb_loops);
+
+		t2 = stress_time_now();
+		total_dur += dur;
+		dur_remainder = total_dur - (t2 - t1);
+
+		if (dur_remainder >= 0.0) {
+			struct timespec t;
+			time_t sec = (time_t)dur_remainder;
+
+			t.tv_sec = sec;
+			t.tv_nsec = (long)((dur_remainder -
+				(double)sec) *
+				STRESS_NANOSECOND);
+			(void)nanosleep(&t, NULL);
+		}
+		if (!keep_stressing_flag())
+			break;
+	}
+	*valid = true;
+	return ((uintptr_t)ptr - (uintptr_t)start) / KB;
+}
+#endif
+
+#if defined(HAVE_ASM_X86_REP_STOSQ)
+static inline void OPTIMIZE3 stress_memrate_stosq(void *ptr, const uint32_t mb_loops)
+{
+	register void *p = ptr;
+	register const uint32_t l = mb_loops;
+
+	__asm__ __volatile__(
+		"mov $0xaaaaaaaaaaaaaaaa,%%rax\n;"
+		"mov %0,%%rdi\n;"
+		"mov %1,%%ecx\n;"
+		"rep stosq %%rax,%%es:(%%rdi);\n"
+		:
+		: "r" (p),
+		  "r" (l)
+		: "ecx","rdi","rax");
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos64(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos(context, valid, stress_memrate_stosq, sizeof(uint64_t));
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos_rate64(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos_rate(context, valid, stress_memrate_stosq, sizeof(uint64_t));
+}
+#endif
+
+#if defined(HAVE_ASM_X86_REP_STOSD)
+static inline void OPTIMIZE3 stress_memrate_stosd(void *ptr, const uint32_t mb_loops)
+{
+	register void *p = ptr;
+	register const uint32_t l = mb_loops;
+
+	__asm__ __volatile__(
+		"mov $0xaaaaaaaa,%%eax\n;"
+		"mov %0,%%rdi\n;"
+		"mov %1,%%ecx\n;"
+		"rep stosl %%eax,%%es:(%%rdi);\n"	/* gcc calls it stosl and not stosw */
+		:
+		: "r" (p),
+		  "r" (l)
+		: "ecx","rdi","eax");
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos32(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos(context, valid, stress_memrate_stosd, sizeof(uint32_t));
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos_rate32(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos_rate(context, valid, stress_memrate_stosd, sizeof(uint32_t));
+}
+#endif
+
+#if defined(HAVE_ASM_X86_REP_STOSW)
+static inline void OPTIMIZE3 stress_memrate_stosw(void *ptr, const uint32_t mb_loops)
+{
+	register void *p = ptr;
+	register const uint32_t l = mb_loops;
+
+	__asm__ __volatile__(
+		"mov $0xaaaa,%%ax\n;"
+		"mov %0,%%rdi\n;"
+		"mov %1,%%ecx\n;"
+		"rep stosw %%ax,%%es:(%%rdi);\n"
+		:
+		: "r" (p),
+		  "r" (l)
+		: "ecx","rdi","ax");
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos16(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos(context, valid, stress_memrate_stosw, sizeof(uint16_t));
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos_rate16(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos_rate(context, valid, stress_memrate_stosw, sizeof(uint16_t));
+}
+#endif
+
+#if defined(HAVE_ASM_X86_REP_STOSB)
+static inline void OPTIMIZE3 stress_memrate_stosb(void *ptr, const uint32_t mb_loops)
+{
+	register void *p = ptr;
+	register const uint32_t l = mb_loops;
+
+	__asm__ __volatile__(
+		"mov $0xaa,%%al\n;"
+		"mov %0,%%rdi\n;"
+		"mov %1,%%ecx\n;"
+		"rep stosb %%al,%%es:(%%rdi);\n"
+		:
+		: "r" (p),
+		  "r" (l)
+		: "ecx","rdi","al");
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos8(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos(context, valid, stress_memrate_stosb, sizeof(uint8_t));
+}
+
+static inline uint64_t OPTIMIZE3 stress_memrate_write_stos_rate8(
+        const stress_memrate_context_t *context,
+        bool *valid)
+{
+	return stress_memrate_stos_rate(context, valid, stress_memrate_stosb, sizeof(uint8_t));
+}
+#endif
+
 #define STRESS_MEMRATE_WRITE_RATE(size, type)			\
 static uint64_t TARGET_CLONES stress_memrate_write_rate##size(	\
 	const stress_memrate_context_t *context,		\
@@ -534,6 +724,18 @@ STRESS_MEMRATE_WRITE(8, uint8_t)
 STRESS_MEMRATE_WRITE_RATE(8, uint8_t)
 
 static stress_memrate_info_t memrate_info[] = {
+#if defined(HAVE_ASM_X86_REP_STOSQ)
+	{ "write64stoq", MR_WR,	stress_memrate_write_stos64,	stress_memrate_write_stos_rate64 },
+#endif
+#if defined(HAVE_ASM_X86_REP_STOSD)
+	{ "write32stow",MR_WR,	stress_memrate_write_stos32,	stress_memrate_write_stos_rate32 },
+#endif
+#if defined(HAVE_ASM_X86_REP_STOSW)
+	{ "write16stod",MR_WR,	stress_memrate_write_stos16,	stress_memrate_write_stos_rate16 },
+#endif
+#if defined(HAVE_ASM_X86_REP_STOSB)
+	{ "write8stob",	MR_WR,	stress_memrate_write_stos8,	stress_memrate_write_stos_rate8 },
+#endif
 #if defined(HAVE_NT_STORE128)
 	{ "write128nt",	MR_WR, stress_memrate_write_nt128,	stress_memrate_write_nt_rate128 },
 #endif
@@ -608,10 +810,8 @@ static inline void *stress_memrate_mmap(const stress_args_t *args, uint64_t sz)
 	} else {
 #if defined(HAVE_MADVISE) &&	\
     defined(MADV_HUGEPAGE)
-		int ret, advice = MADV_NORMAL;
 
-		ret = madvise(ptr, sz, advice);
-		(void)ret;
+		VOID_RET(int, madvise(ptr, sz, MADV_HUGEPAGE));
 #endif
 	}
 	return ptr;
@@ -679,7 +879,6 @@ static int stress_memrate(const stress_args_t *args)
 {
 	int rc;
 	size_t i, stats_size;
-	bool lock = false;
 	stress_memrate_context_t context;
 
 	context.memrate_bytes = DEFAULT_MEMRATE_BYTES;
@@ -695,8 +894,11 @@ static int stress_memrate(const stress_args_t *args)
 
 	context.stats = (stress_memrate_stats_t *)mmap(NULL, stats_size,
 		PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
-	if (context.stats == MAP_FAILED)
+	if (context.stats == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %zd byte statistics buffer, skipping stressor\n",
+			args->name, stats_size);
 		return EXIT_NO_RESOURCE;
+	}
 	for (i = 0; i < memrate_items; i++) {
 		context.stats[i].duration = 0.0;
 		context.stats[i].kbytes = 0.0;
@@ -711,7 +913,7 @@ static int stress_memrate(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	pr_lock(&lock);
+	pr_lock();
 	for (i = 0; i < memrate_items; i++) {
 		if (!context.stats[i].valid)
 			continue;
@@ -719,17 +921,14 @@ static int stress_memrate(const stress_args_t *args)
 			char tmp[32];
 			const double rate = context.stats[i].kbytes / (context.stats[i].duration * KB);
 
-			pr_inf_lock(&lock, "%s: %10.10s: %12.2f MB/sec\n",
-				args->name, memrate_info[i].name, rate);
-
-			(void)snprintf(tmp, sizeof(tmp), "%s MB/sec", memrate_info[i].name);
-			stress_misc_stats_set(args->misc_stats, i, tmp, rate);
+			(void)snprintf(tmp, sizeof(tmp), "%s MB per sec", memrate_info[i].name);
+			stress_metrics_set(args, i, tmp, rate);
 		} else {
-			pr_inf_lock(&lock, "%s: %10.10s: interrupted early\n",
+			pr_inf("%s: %10.10s: interrupted early\n",
 				args->name, memrate_info[i].name);
 		}
 	}
-	pr_unlock(&lock);
+	pr_unlock();
 
 	(void)munmap((void *)context.stats, stats_size);
 
