@@ -459,10 +459,17 @@ typedef struct {
 #define STRESS_MISC_METRICS_MAX	(24)
 
 typedef struct {
-	char *description;
-	double value;
+	void *lock;			/* optional lock */
+	double	duration;		/* time per op */
+	double	count;			/* number of ops */
+	volatile double	t_start;	/* optional start time */
 } stress_metrics_t;
 
+typedef struct {
+	char *description;		/* description of metric */
+	double value;			/* value of metric */
+} stress_metrics_data_t;
+
 /* stressor args */
 typedef struct {
 	uint64_t *counter;		/* stressor counter */
@@ -474,7 +481,7 @@ typedef struct {
 	pid_t pid;			/* stressor pid */
 	size_t page_size;		/* page size */
 	stress_mapped_t *mapped;	/* mmap'd pages, addr of g_shared mapped */
-	stress_metrics_t *metrics;	/* misc per stressor metrics */
+	stress_metrics_data_t *metrics;	/* misc per stressor metrics */
 	const struct stressor_info *info; /* stressor info */
 } stress_args_t;
 
@@ -869,7 +876,7 @@ typedef struct {
 	stress_tz_t tz;			/* thermal zones */
 #endif
 	stress_checksum_t *checksum;	/* pointer to checksum data */
-	stress_metrics_t metrics[STRESS_MISC_METRICS_MAX];
+	stress_metrics_data_t metrics[STRESS_MISC_METRICS_MAX];
 #if defined(HAVE_GETRUSAGE)
 	double rusage_utime;		/* rusage user time */
 	double rusage_stime;		/* rusage system time */
@@ -1397,6 +1404,7 @@ typedef enum {
 	OPT_ipsec_mb_ops,
 	OPT_ipsec_mb_feature,
 	OPT_ipsec_mb_jobs,
+	OPT_ipsec_mb_method,
 
 	OPT_itimer,
 	OPT_itimer_ops,
@@ -1725,8 +1733,9 @@ typedef enum {
 	OPT_poll_fds,
 
 	OPT_prefetch,
-	OPT_prefetch_ops,
 	OPT_prefetch_l3_size,
+	OPT_prefetch_method,
+	OPT_prefetch_ops,
 
 	OPT_prctl,
 	OPT_prctl_ops,
@@ -2125,6 +2134,7 @@ typedef enum {
 
 	OPT_tsc,
 	OPT_tsc_ops,
+	OPT_tsc_lfence,
 
 	OPT_tsearch,
 	OPT_tsearch_ops,
@@ -2147,6 +2157,9 @@ typedef enum {
 	OPT_udp_flood_domain,
 	OPT_udp_flood_if,
 
+	OPT_umount,
+	OPT_umount_ops,
+
 	OPT_unshare,
 	OPT_unshare_ops,
 
@@ -2227,6 +2240,9 @@ typedef enum {
 	OPT_wait,
 	OPT_wait_ops,
 
+	OPT_waitcpu,
+	OPT_waitcpu_ops,
+
 	OPT_watchdog,
 	OPT_watchdog_ops,
 
