@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -129,7 +129,7 @@ static int stress_radixsort(const stress_args_t *args)
 	/* This is very expensive, do it once */
 	for (ptr = text, i = 0; i < n; i++, ptr += STR_SIZE) {
 		data[i] = ptr;
-		stress_strnrnd((char *)ptr, STR_SIZE);
+		stress_rndstr((char *)ptr, STR_SIZE);
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -167,7 +167,7 @@ static int stress_radixsort(const stress_args_t *args)
 
 		/* Randomize first char */
 		for (ptr = text, i = 0; i < n; i++, ptr += STR_SIZE)
-			*ptr = 'a' + (stress_mwc8() % 26);
+			*ptr = 'a' + stress_mwc8modn(26);
 
 		inc_counter(args);
 	} while (keep_stressing(args));
