@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-capabilities.h"
+#include "core-net.h"
 
 #if defined(HAVE_LINUX_SOCKIOS_H)
 #include <linux/sockios.h>
@@ -28,15 +29,38 @@
 #include <netinet/ip.h>
 #endif
 
+#define MIN_RAWSOCK_PORT		(1024)
+#define MAX_RAWSOCK_PORT		(65535)
+#define DEFAULT_RAWSOCK_PORT		(45000)
+
 static const stress_help_t help[] = {
 	{ NULL,	"rawsock N",		"start N workers performing raw socket send/receives " },
 	{ NULL,	"rawsock-ops N",	"stop after N raw socket bogo operations" },
+	{ NULL,	"rawsock-port P",	"use socket P to P + number of workers - 1" },
 	{ NULL,	NULL,			NULL }
 };
 
+/*
+ *  stress_set_rawsock_port()
+ *	set port to use
+ */
+static int stress_set_rawsock_port(const char *opt)
+{
+	int rawsock_port;
+
+	stress_set_net_port("rawsock-port", opt,
+		MIN_RAWSOCK_PORT, MAX_RAWSOCK_PORT, &rawsock_port);
+	return stress_set_setting("rawsock-port", TYPE_ID_INT, &rawsock_port);
+}
+
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_rawsock_port,	stress_set_rawsock_port },
+	{ 0,			NULL },
+};
+
 #if defined(SOCK_RAW) &&	\
     defined(IPPROTO_RAW) &&	\
-    defined(HAVE_ICMPHDR) &&	\
+    defined(HAVE_IPHDR) &&	\
     defined(__linux__)
 
 typedef struct {
@@ -78,12 +102,27 @@ static int stress_rawsock_supported(const char *name)
 static int stress_rawsock(const stress_args_t *args)
 {
 	pid_t pid;
-	int rc = EXIT_SUCCESS;
+	int rc = EXIT_SUCCESS, reserved_port;
+	double t_start, duration = 0.0, bytes = 0.0, rate;
+	int rawsock_port = DEFAULT_RAWSOCK_PORT;
 
 	if (!rawsock_lock) {
-		pr_inf("%s: failed to create rawsock lock, skipping stressor\n", args->name);
+		pr_inf_skip("%s: failed to create rawsock lock, skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
+	(void)stress_get_setting("rawsock-port", &rawsock_port);
+
+	rawsock_port += args->instance;
+	reserved_port = stress_net_reserve_ports(rawsock_port, rawsock_port);
+	if (reserved_port < 0) {
+		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
+			args->name, rawsock_port);
+		return EXIT_NO_RESOURCE;
+	}
+	rawsock_port = reserved_port;
+
+	pr_dbg("%s: process [%d] using socket port %d\n",
+		args->name, (int)args->pid, rawsock_port);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
@@ -117,7 +156,7 @@ again:
 
 		(void)memset(&addr, 0, sizeof(addr));
 		addr.sin_family = AF_INET;
-		addr.sin_port = 45000;
+		addr.sin_port = rawsock_port;
 		addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
 
 		(void)memset(&pkt, 0, sizeof(pkt));
@@ -199,6 +238,7 @@ again:
 		g_shared->rawsock.ready++;
 		(void)stress_lock_release(rawsock_lock);
 
+		t_start = stress_time_now();
 		while (keep_stressing(args)) {
 			stress_raw_packet_t ALIGN64 pkt;
 			socklen_t len = sizeof(addr);
@@ -213,8 +253,9 @@ again:
 					pr_fail("%s: recvfrom failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 				break;
+			} else  {
+				bytes += n;
 			}
-
 #if defined(SIOCINQ)
 			/* Occasionally exercise SIOCINQ */
 			if ((pkt.data & 0xff) == 0) {
@@ -228,7 +269,9 @@ again:
 #endif
 			inc_counter(args);
 		}
-
+		duration = stress_time_now() - t_start;
+		rate = (duration > 0.0) ? bytes / duration : 0.0;
+		stress_metrics_set(args, 0, "MB recv'd per sec", rate / (double)MB);
 die:
 		if (pid) {
 			(void)kill(pid, SIGKILL);
@@ -240,6 +283,7 @@ die:
 	}
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_net_release_ports(rawsock_port, rawsock_port);
 
 	return rc;
 }
@@ -247,6 +291,7 @@ finish:
 stressor_info_t stress_rawsock_info = {
 	.stressor = stress_rawsock,
 	.class = CLASS_NETWORK | CLASS_OS,
+	.opt_set_funcs = opt_set_funcs,
 	.supported = stress_rawsock_supported,
 	.help = help,
 	.init = stress_rawsock_init,
@@ -254,8 +299,10 @@ stressor_info_t stress_rawsock_info = {
 };
 #else
 stressor_info_t stress_rawsock_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK | CLASS_OS,
-	.help = help
+	.opt_set_funcs = opt_set_funcs,
+	.help = help,
+	.unimplemented_reason = "built without linux/sockios.h, SOCK_RAW, IPPROTO_RAW or only supported on Linux"
 };
 #endif
