@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -58,7 +58,6 @@ static void MLOCKED_TEXT stress_sigio_handler(int signum)
 
 	async_sigs++;
 
-
 	if (rd_fd > 0) {
 		/*
 		 *  Data is ready, so drain as much as possible
@@ -88,10 +87,10 @@ static void MLOCKED_TEXT stress_sigio_handler(int signum)
 static int stress_sigio(const stress_args_t *args)
 {
 	int ret, rc = EXIT_FAILURE, fds[2], status, flags = -1;
+	double t_start, t_delta, rate;
 
 	rd_fd = -1;
 	sigio_args = args;
-	double t_start, t_delta;
 	pid = -1;
 
 	time_end = stress_time_now() + (double)g_opt_timeout;
@@ -140,7 +139,6 @@ again:
 		/* Child */
 		char buffer[BUFFER_SIZE];
 
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -150,6 +148,8 @@ again:
 		(void)close(fds[0]);
 		(void)memset(buffer, 0, sizeof buffer);
 
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 		while (keep_stressing(args)) {
 			ssize_t n;
 
@@ -192,15 +192,12 @@ again:
 	} while (keep_stressing(args));
 
 	t_delta = stress_time_now() - t_start;
-
-	if (t_delta > 0.0) 
-		pr_inf("%s: %.2f SIGIO signals/sec\n",
-			args->name, (double)async_sigs / t_delta);
+	rate = (t_delta > 0.0) ? (double)async_sigs / t_delta : 0.0;
+	stress_metrics_set(args, 0, "SIGIO signals per sec", rate);
 
 finish:
 	/*  And ignore IO signals from now on */
-	ret = stress_sighandler(args->name, SIGIO, SIG_IGN, NULL);
-	(void)ret;
+	VOID_RET(int, stress_sighandler(args->name, SIGIO, SIG_IGN, NULL));
 
 	rc = EXIT_SUCCESS;
 reap:
@@ -211,8 +208,7 @@ reap:
 
 err:
 	if (flags != -1) {
-		ret = fcntl(fds[0], F_SETFL, flags & ~(O_ASYNC | O_NONBLOCK));
-		(void)ret;
+		VOID_RET(int, fcntl(fds[0], F_SETFL, flags & ~(O_ASYNC | O_NONBLOCK)));
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -232,8 +228,9 @@ stressor_info_t stress_sigio_info = {
 };
 #else
 stressor_info_t stress_sigio_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without fcntl() commands O_ASYNC, O_NONBLOCK, F_SETOWN, F_GETFL or F_SETFL"
 };
 #endif
