@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -104,17 +104,19 @@ static int stress_swap_supported(const char *name)
  */
 static int stress_swapoff(const char *path)
 {
-	int ret, i;
+	int i;
 
 	for (i = 0; i < 25; i++) {
+		int ret;
+
 		errno = 0;
 		ret = swapoff(path);
 		if (ret == 0)
-			break;
+			return ret;
 		if ((ret < 0) && (errno != EINTR))
 			break;
 	}
-	return ret;
+	return -1;
 }
 
 static int stress_swap_zero(
@@ -144,7 +146,7 @@ static int stress_swap_zero(
 static int stress_swap_set_size(
 	const stress_args_t *args,
 	const int fd,
-	const size_t npages,
+	const uint32_t npages,
 	const int bad_flags)
 {
 	char signature[] = SWAP_SIGNATURE;
@@ -172,22 +174,18 @@ static int stress_swap_set_size(
 		"SNG-SWP-%" PRIx32, args->instance);
 
 	if (bad_flags & SWAP_HDR_BAD_VERSION)
-		swap_info.version = ~SWAP_VERSION;	/* Invalid */
+		swap_info.version = (uint32_t)~SWAP_VERSION;	/* Invalid */
 	else
 		swap_info.version = SWAP_VERSION;
 
+	swap_info.last_page = npages - 1;		/* default */
 	if (bad_flags & SWAP_HDR_ZERO_LAST_PAGE)
 		swap_info.last_page = 0;		/* Invalid */
-	else
-		swap_info.last_page = npages - 1;
-
-	if (bad_flags & SWAP_HDR_BAD_LAST_PAGE)
+	else if (bad_flags & SWAP_HDR_BAD_LAST_PAGE)
 		swap_info.last_page = npages + 1;	/* Invalid */
-	else
-		swap_info.last_page = npages - 1;
 
 	if (bad_flags & SWAP_HDR_BAD_NR_BAD)
-		swap_info.nr_badpages = ~0;		/* Dire */
+		swap_info.nr_badpages = ~0U;		/* Dire */
 	else
 		swap_info.nr_badpages = 0;
 
@@ -212,23 +210,25 @@ static int stress_swap_set_size(
 static void stress_swap_check_swapped(
 	void *addr,
 	const size_t page_size,
-	const size_t npages,
+	const uint32_t npages,
 	uint64_t *swapped_out,
 	uint64_t *swapped_total)
 {
 	unsigned char *vec;
 	register size_t n = 0;
-	register size_t i;
 
 	*swapped_total += npages;
 
-	vec = calloc(npages, sizeof(*vec));
+	vec = calloc((size_t)npages, sizeof(*vec));
 	if (!vec)
 		return;
 
-	shim_mincore(addr, page_size * npages, vec);
-	for (i = 0; i < npages; i++)
-		n += ((vec[i] & 1) == 0);
+	if (shim_mincore(addr, page_size * (size_t)npages, vec) == 0) {
+		register uint32_t i;
+
+		for (i = 0; i < npages; i++)
+			n += ((vec[i] & 1) == 0);
+	}
 
 	*swapped_out += n;
 	free(vec);
@@ -250,7 +250,7 @@ static int stress_swap(const stress_args_t *args)
 
 	page = calloc(1, page_size);
 	if (!page) {
-		pr_err("%s: failed to allocate 1 page: errno=%d (%s)\n",
+		pr_inf_skip("%s: failed to allocate 1 page: errno=%d (%s), skipping stressor\n",
 			args->name, errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto tidy_ret;
@@ -258,7 +258,7 @@ static int stress_swap(const stress_args_t *args)
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
-		ret = exit_status(-ret);
+		ret = stress_exit_status(-ret);
 		goto tidy_free;
 	}
 
@@ -266,7 +266,7 @@ static int stress_swap(const stress_args_t *args)
 		filename, sizeof(filename), stress_mwc32());
 	fd = open(filename, O_CREAT | O_RDWR, S_IRUSR);
 	if (fd < 0) {
-		ret = exit_status(errno);
+		ret = stress_exit_status(errno);
 		pr_fail("%s: open swap file %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto tidy_rm;
@@ -283,8 +283,7 @@ static int stress_swap(const stress_args_t *args)
 		 */
 		if (ioctl(fd, SHIM_EXT2_IOC_GETFLAGS, &flags) == 0) {
 			flags |= SHIM_FS_NOCOW_FL;
-			ret = ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &flags);
-			(void)ret;
+			VOID_RET(int, ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &flags));
 		}
 	}
 #endif
@@ -300,9 +299,9 @@ static int stress_swap(const stress_args_t *args)
 		int swapflags = 0;
 		int bad_flags;
 		char *ptr;
-		size_t npages = (size_t)(stress_mwc32() % (MAX_SWAP_PAGES - MIN_SWAP_PAGES)) +
+		uint32_t npages = stress_mwc32modn(MAX_SWAP_PAGES - MIN_SWAP_PAGES) +
 				  MIN_SWAP_PAGES;
-		const size_t mmap_size = npages * page_size;
+		const size_t mmap_size = (size_t)npages * page_size;
 
 #if defined(SWAP_FLAG_PREFER)
 		if (stress_mwc1()) {
@@ -316,7 +315,7 @@ static int stress_swap(const stress_args_t *args)
 #endif
 		/* Periodically create bad swap header */
 		if (stress_mwc8() < 16) {
-			const size_t idx = stress_mwc8() % SIZEOF_ARRAY(bad_header_flags);
+			const size_t idx = stress_mwc8modn(SIZEOF_ARRAY(bad_header_flags));
 			bad_flags = bad_header_flags[idx];
 		} else {
 			bad_flags = SWAP_HDR_SANE;	/* No bad header */
@@ -359,9 +358,9 @@ static int stress_swap(const stress_args_t *args)
 
 			/* Add simple check value to start of each page */
 			for (i = 0, p = ptr; p < p_end; p += page_size, i++) {
-				uintptr_t *up = (uintptr_t *)p;
+				uintptr_t *up = (uintptr_t *)(uintptr_t)p;
 
-				(void)memset(p, i, page_size);
+				(void)memset(p, (int)i, page_size);
 				*up = (uintptr_t)p;
 			}
 #if defined(MADV_PAGEOUT)
@@ -372,12 +371,12 @@ static int stress_swap(const stress_args_t *args)
 
 			/* Check page has check address value */
 			for (i = 0, p = ptr; p < p_end; p += page_size, i++) {
-				uintptr_t *up = (uintptr_t *)p;
+				uintptr_t *up = (uintptr_t *)(uintptr_t)p;
 
 				if (*up != (uintptr_t)p) {
 					pr_fail("%s: failed: address %p contains "
 						"%" PRIuPTR " and not %" PRIuPTR "\n",
-						args->name, p, *up, (uintptr_t)p);
+						args->name, (void *)p, *up, (uintptr_t)p);
 				}
 			}
 			(void)munmap(ptr, mmap_size);
@@ -394,20 +393,17 @@ static int stress_swap(const stress_args_t *args)
 		/* Exercise illegal swapon filename */
 		ret = swapon("", swapflags);
 		if (ret == 0)
-			ret = stress_swapoff("");	/* Should never happen */
-		(void)ret;
+			VOID_RET(int, stress_swapoff(""));	/* Should never happen */
 
 		/* Exercise illegal swapoff filename */
 		ret = stress_swapoff("");
 		if (ret == 0)
-			ret = swapon("", swapflags);	/* Should never happen */
-		(void)ret;
+			VOID_RET(int, swapon("", swapflags));	/* Should never happen */
 
 		/* Exercise illegal swapon flags */
 		ret = swapon(filename, ~0);
 		if (ret == 0)
-			ret = stress_swapoff(filename);	/* Should never happen */
-		(void)ret;
+			VOID_RET(int, stress_swapoff(filename));/* Should never happen */
 
 		inc_counter(args);
 	} while (keep_stressing(args));
@@ -442,8 +438,9 @@ stressor_info_t stress_swap_info = {
 };
 #else
 stressor_info_t stress_swap_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/swap.h or swap() system call"
 };
 #endif
