@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -68,14 +68,12 @@ again:
 		return -1;
 	}
 	if (pid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
 		func(args, fds);
 		_exit(EXIT_SUCCESS);
 	}
-	(void)setpgid(pid, g_pgrp);
 	return pid;
 }
 
@@ -85,7 +83,7 @@ again:
  */
 static void stress_tee_pipe_write(const stress_args_t *args, int fds[2])
 {
-	static uint64_t buffer[TEE_IO_SIZE / sizeof(uint64_t)];
+	static uint64_t ALIGN64 buffer[TEE_IO_SIZE / sizeof(uint64_t)];
 	uint64_t counter = 0;
 
 	(void)args;
@@ -113,22 +111,23 @@ static void stress_tee_pipe_write(const stress_args_t *args, int fds[2])
  */
 static void stress_tee_pipe_read(const stress_args_t *args, int fds[2])
 {
-	static uint64_t buffer[TEE_IO_SIZE / sizeof(uint64_t)];
+	static uint64_t ALIGN64 buffer[TEE_IO_SIZE / sizeof(uint64_t)];
 	uint64_t count = 0;
 
 	(void)close(fds[1]);
 
 	while (keep_stressing_flag()) {
-		ssize_t ret;
 		size_t n = 0;
 
 		while (n < sizeof(buffer)) {
+			ssize_t ret;
+
 			ret = read(fds[0], buffer, sizeof(buffer));
 			if (ret < 0) {
 				if (errno != EAGAIN)
 					break;
 			} else {
-				n += ret;
+				n += (size_t)ret;
 			}
 		}
 		if (buffer[0] != sizeof(buffer)) {
@@ -207,6 +206,7 @@ static int stress_tee(const stress_args_t *args)
 	pid_t pids[2];
 	int ret = EXIT_FAILURE, status;
 	const int release = stress_get_kernel_release();
+	double duration = 0.0, bytes = 0.0, rate;
 
 	if (stress_sighandler(args->name, SIGPIPE, stress_sigpipe_handler, NULL) < 0)
 		return EXIT_NO_RESOURCE;
@@ -232,10 +232,11 @@ static int stress_tee(const stress_args_t *args)
 		goto tidy_child1;
 	(void)close(pipe_out[0]);
 
-
 	do {
-		len = tee(pipe_in[0], pipe_out[1],
-			INT_MAX, 0 & SPLICE_F_NONBLOCK);
+		double t;
+
+		t = stress_time_now();
+		len = tee(pipe_in[0], pipe_out[1], INT_MAX, 0);
 
 		if (len < 0) {
 			if (errno == EAGAIN)
@@ -254,6 +255,8 @@ static int stress_tee(const stress_args_t *args)
 		} else {
 			if (len == 0)
 				break;
+			duration += stress_time_now() - t;
+			bytes += (double)len;
 		}
 
 		while (len > 0) {
@@ -277,6 +280,9 @@ static int stress_tee(const stress_args_t *args)
 
 	ret = EXIT_SUCCESS;
 
+	rate = (duration > 0.0) ? bytes / duration : 0.0;
+	stress_metrics_set(args, 0, "MB per sec tee rate", rate / (double)MB);
+
 tidy_child2:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(pipe_out[1]);
@@ -302,8 +308,9 @@ stressor_info_t stress_tee_info = {
 };
 #else
 stressor_info_t stress_tee_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without tee() system call or undefined SPLICE_F_NONBLOCK"
 };
 #endif
