@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2022 Colin Ian King
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,6 +46,9 @@ typedef int8_t stress_vint8w256_t	__attribute__ ((vector_size(256 / 8)));
 typedef int8_t stress_vint8w128_t	__attribute__ ((vector_size(128 / 8)));
 typedef int8_t stress_vint8w64_t	__attribute__ ((vector_size(64 / 8)));
 typedef int8_t stress_vint8w32_t	__attribute__ ((vector_size(32 / 8)));
+#if VERY_SMALL
+typedef int8_t stress_vint8w16_t	__attribute__ ((vector_size(16 / 8)));
+#endif
 
 #if VERY_WIDE
 #define VEC_MAX_SZ	sizeof(stress_vint8w8192_t)
@@ -69,12 +72,19 @@ typedef struct {
 	stress_vecwide_func_t	vecwide_func;
 	size_t byte_size;
 	double duration;
+	double count;
 } stress_vecwide_funcs_t;
 
 #define STRESS_VECWIDE(name, type)				\
-static void TARGET_CLONES OPTIMIZE3 name (vec_args_t *vec_args) \
+static void TARGET_CLONES OPTIMIZE3 UNROLL name (vec_args_t *vec_args) \
 {								\
-	type a, b, c, s, v23, v3, res;				\
+	type ALIGN64 a;						\
+	type ALIGN64 b;						\
+	type ALIGN64 c;						\
+	type ALIGN64 s;						\
+	type ALIGN64 v23;					\
+	type ALIGN64 v3;					\
+	type ALIGN64 res;					\
 	register int i;						\
 								\
 	(void)memcpy(&a, vec_args->a, sizeof(a));		\
@@ -97,7 +107,7 @@ static void TARGET_CLONES OPTIMIZE3 name (vec_args_t *vec_args) \
 	res = a + b + c;					\
 								\
 	for (i = 0; i < (int)sizeof(res); i++) {		\
-		stress_uint8_put(res[i]);			\
+		stress_uint8_put((uint8_t)res[i]);		\
 	}							\
 }
 
@@ -112,19 +122,25 @@ STRESS_VECWIDE(stress_vecwide_256, stress_vint8w256_t)
 STRESS_VECWIDE(stress_vecwide_128, stress_vint8w128_t)
 STRESS_VECWIDE(stress_vecwide_64, stress_vint8w64_t)
 STRESS_VECWIDE(stress_vecwide_32, stress_vint8w32_t)
+#if VERY_SMALL
+STRESS_VECWIDE(stress_vecwide_16, stress_vint8w16_t)
+#endif
 
 static stress_vecwide_funcs_t stress_vecwide_funcs[] = {
 #if VERY_WIDE
-	{ stress_vecwide_8192, sizeof(stress_vint8w8192_t), 0.0 },
-	{ stress_vecwide_4096, sizeof(stress_vint8w4096_t), 0.0 },
+	{ stress_vecwide_8192, sizeof(stress_vint8w8192_t), 0.0, 0.0 },
+	{ stress_vecwide_4096, sizeof(stress_vint8w4096_t), 0.0, 0.0 },
+#endif
+	{ stress_vecwide_2048, sizeof(stress_vint8w2048_t), 0.0, 0.0 },
+	{ stress_vecwide_1024, sizeof(stress_vint8w1024_t), 0.0, 0.0 },
+	{ stress_vecwide_512,  sizeof(stress_vint8w512_t),  0.0, 0.0 },
+	{ stress_vecwide_256,  sizeof(stress_vint8w256_t),  0.0, 0.0 },
+	{ stress_vecwide_128,  sizeof(stress_vint8w128_t),  0.0, 0.0 },
+	{ stress_vecwide_64,   sizeof(stress_vint8w64_t),   0.0, 0.0 },
+	{ stress_vecwide_32,   sizeof(stress_vint8w32_t),   0.0, 0.0 },
+#if VERY_SMALL
+	{ stress_vecwide_16,   sizeof(stress_vint8w16_t),   0.0, 0.0 },
 #endif
-	{ stress_vecwide_2048, sizeof(stress_vint8w2048_t), 0.0 },
-	{ stress_vecwide_1024, sizeof(stress_vint8w1024_t), 0.0 },
-	{ stress_vecwide_512,  sizeof(stress_vint8w512_t),  0.0 },
-	{ stress_vecwide_256,  sizeof(stress_vint8w256_t),  0.0 },
-	{ stress_vecwide_128,  sizeof(stress_vint8w128_t),  0.0 },
-	{ stress_vecwide_64,   sizeof(stress_vint8w64_t),   0.0 },
-	{ stress_vecwide_32,   sizeof(stress_vint8w32_t),   0.0 },
 };
 
 static int stress_vecwide(const stress_args_t *args)
@@ -138,16 +154,18 @@ static int stress_vecwide(const stress_args_t *args)
 	vec_args = (vec_args_t *)mmap(NULL, vec_args_size, PROT_READ | PROT_WRITE,
 					MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (vec_args == MAP_FAILED) {
-		pr_inf("%s: skipping stressor, failed to allocate vectors, errno=%d (%s)\n",
+		pr_inf_skip("%s: skipping stressor, failed to allocate vectors, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
-	for (i = 0; i < SIZEOF_ARRAY(stress_vecwide_funcs); i++)
+	for (i = 0; i < SIZEOF_ARRAY(stress_vecwide_funcs); i++) {
 		stress_vecwide_funcs[i].duration = 0.0;
+		stress_vecwide_funcs[i].count = 0.0;
+	}
 
 	for (i = 0; i < SIZEOF_ARRAY(vec_args->a); i++) {
-		vec_args->a[i] = (int8_t)i;
+		vec_args->a[i] = (uint8_t)i;
 		vec_args->b[i] = stress_mwc8();
 		vec_args->c[i] = stress_mwc8();
 		vec_args->s[i] = stress_mwc8();
@@ -169,6 +187,7 @@ static int stress_vecwide(const stress_args_t *args)
 
 			total_duration += dt;
 			stress_vecwide_funcs[i].duration += dt;
+			stress_vecwide_funcs[i].count += 1.0;
 
 			inc_counter(args);
 		}
@@ -179,10 +198,8 @@ static int stress_vecwide(const stress_args_t *args)
 	}
 
 	if (args->instance == 0) {
-		bool lock = false;
-
-		pr_lock(&lock);
-		pr_dbg_lock(&lock, "Bytes %% dur  %% exp (x win) (> 1.0 is better than expected)\n");
+		pr_lock();
+		pr_dbg("%s: Bits  %% Dur  %% Exp (x Win) (> 1.0 is better than expected)\n", args->name);
 		for (i = 0; i < SIZEOF_ARRAY(stress_vecwide_funcs); i++) {
 			double dur_pc, exp_pc, win;
 
@@ -190,11 +207,22 @@ static int stress_vecwide(const stress_args_t *args)
 			exp_pc = (double)stress_vecwide_funcs[i].byte_size / (double)total_bytes * 100.0;
 			win    = exp_pc / dur_pc;
 
-			pr_dbg_lock(&lock, "%5zd %5.2f%% %5.2f%% %5.2f\n",
-				stress_vecwide_funcs[i].byte_size,
+			pr_dbg("%s: %5zd %5.2f%% %5.2f%% %5.2f\n",
+				args->name, 8 * stress_vecwide_funcs[i].byte_size,
 				dur_pc, exp_pc, win);
 		}
-		pr_unlock(&lock);
+		pr_dbg("%s: Key: Bits = vector width in bits, Dur = %% total run time,\n", args->name);
+		pr_dbg("%s       Exp = %% expected run time, Win = performance gain\n", args->name);
+		pr_unlock();
+	}
+
+	for (i = 0; i < SIZEOF_ARRAY(stress_vecwide_funcs); i++) {
+		char str[64];
+		const double rate = (stress_vecwide_funcs[i].duration > 0) ?
+				stress_vecwide_funcs[i].count / stress_vecwide_funcs[i].duration : 0.0;
+
+		(void)snprintf(str, sizeof(str), "vecwide%zd ops per sec", stress_vecwide_funcs[i].byte_size * 8);
+		stress_metrics_set(args, i, str, rate);
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -211,8 +239,9 @@ stressor_info_t stress_vecwide_info = {
 };
 #else
 stressor_info_t stress_vecwide_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_CPU_CACHE,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without compiler support for vector data/operations"
 };
 #endif
