@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,8 +38,8 @@ static int stress_yield(const stress_args_t *args)
 	uint64_t max_ops_per_yielder;
 	size_t counters_sz;
 	int32_t cpus = stress_get_processors_configured();
-	const size_t instances = args->num_instances;
-	size_t yielders = 2;
+	const uint32_t instances = args->num_instances;
+	uint32_t yielders = 2;
 #if defined(HAVE_SCHED_GETAFFINITY)
 	cpu_set_t mask;
 #endif
@@ -56,8 +56,8 @@ static int stress_yield(const stress_args_t *args)
 			PRId32 " yielder%s (instance %" PRIu32 ")\n",
 			args->name, cpus, (cpus == 1) ? "" : "s", args->instance);
 	} else {
-		if (CPU_COUNT(&mask) < cpus)
-			cpus = CPU_COUNT(&mask);
+		if (CPU_COUNT(&mask) < (int)cpus)
+			cpus = (int32_t)CPU_COUNT(&mask);
 		pr_inf("%s: limiting to %" PRId32 " child yielder%s (instance %"
 			PRIu32 ")\n", args->name, cpus, (cpus == 1) ? "" : "s", args->instance);
 	}
@@ -71,11 +71,12 @@ static int stress_yield(const stress_args_t *args)
 	 */
 	if (cpus > 0) {
 		cpus *= 2;
-		yielders = (size_t)cpus / instances;
+		yielders = cpus / instances;
 		if (yielders < 1)
 			yielders = 1;
 		if (!args->instance) {
-			size_t residual = (size_t)cpus - (yielders * instances);
+			/* residual may be -ve, ensure it is signed */
+			int32_t residual = cpus - (int32_t)(yielders * instances);
 
 			if (residual > 0)
 				yielders += residual;
@@ -85,7 +86,8 @@ static int stress_yield(const stress_args_t *args)
 	max_ops_per_yielder = args->max_ops / yielders;
 	pids = calloc(yielders, sizeof(*pids));
 	if (!pids) {
-		pr_err("%s: calloc failed\n", args->name);
+		pr_inf_skip("%s: calloc failed allocating %" PRIu32
+			" pids, skipping stressor\n", args->name, yielders);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -93,7 +95,7 @@ static int stress_yield(const stress_args_t *args)
 	counters = (uint64_t *)mmap(NULL, counters_sz, PROT_READ | PROT_WRITE,
 		MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (counters == MAP_FAILED) {
-		int rc = exit_status(errno);
+		int rc = stress_exit_status(errno);
 
 		pr_err("%s: mmap failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -111,7 +113,6 @@ static int stress_yield(const stress_args_t *args)
 				", yielder %zd): errno=%d (%s)\n",
 				args->name, args->instance, i, errno, strerror(errno));
 		} else if (pids[i] == 0) {
-			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
@@ -129,15 +130,12 @@ static int stress_yield(const stress_args_t *args)
 	}
 
 	do {
-		int ret;
-
 		set_counter(args, 0);
 #if defined(__FreeBSD__)
-		ret = shim_sched_yield();
+		VOID_RET(int, shim_sched_yield());
 #else
-		ret = shim_usleep(100000);
+		VOID_RET(int, shim_usleep(100000));
 #endif
-		(void)ret;
 
 		for (i = 0; i < yielders; i++)
 			add_counter(args, counters[i]);
@@ -170,8 +168,9 @@ stressor_info_t stress_yield_info = {
 };
 #else
 stressor_info_t stress_yield_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without scheduling support"
 };
 #endif
