@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-asm-x86.h"
 #include "core-cpu.h"
 #include "core-target-clones.h"
 
@@ -269,7 +270,7 @@ static void stress_rand_data_text(
 	const size_t size = (size_t)((uintptr_t)data_end - (uintptr_t)data);
 
 	(void)args;
-	stress_strnrnd((char *)data, size);
+	stress_rndstr((char *)data, size);
 }
 
 /*
@@ -472,23 +473,7 @@ static void TARGET_CLONES stress_rand_data_fixed(
 	}
 }
 
-#if defined(HAVE_ASM_X86_RDRAND) &&		\
-    defined(STRESS_ARCH_X86_64)
-/*
- *  rdrand64()
- *	read 64 bit random value
- */
-static inline uint64_t rand64(void)
-{
-	uint64_t        ret;
-
-	__asm__ __volatile__("1:;\n\
-	rdrand %0;\n\
-	jnc 1b;\n":"=r"(ret));
-
-	return ret;
-}
-
+#if defined(HAVE_ASM_X86_RDRAND)
 /*
  *  stress_rand_data_rdrand()
  *	fill buffer with data from x86 rdrand instruction
@@ -505,10 +490,10 @@ static void stress_rand_data_rdrand(
 
 	if (stress_cpu_x86_has_rdrand()) {
 		while (ptr < end) {
-			*(ptr++) = rand64();
-			*(ptr++) = rand64();
-			*(ptr++) = rand64();
-			*(ptr++) = rand64();
+			*(ptr++) = stress_asm_x86_rdrand();
+			*(ptr++) = stress_asm_x86_rdrand();
+			*(ptr++) = stress_asm_x86_rdrand();
+			*(ptr++) = stress_asm_x86_rdrand();
 		}
 	} else {
 		while (ptr < end) {
@@ -615,6 +600,29 @@ static void TARGET_CLONES stress_rand_data_gray(
 	val = v;
 }
 
+/*
+ *  stress_rand_data_gray()
+ *	fill buffer with incrementing 16 bit values
+ *
+ */
+static void TARGET_CLONES stress_rand_data_inc16(
+	const stress_args_t *args,
+	uint64_t *RESTRICT data,
+	uint64_t *RESTRICT data_end)
+{
+	register uint16_t val = stress_mwc16();
+	register uint16_t *ptr = (uint16_t *)data;
+	register const uint16_t *end = (uint16_t *)data_end;
+
+	(void)args;
+
+	while (ptr < end) {
+		*(ptr++) = val++;
+		*(ptr++) = val++;
+		*(ptr++) = val++;
+		*(ptr++) = val++;
+	}
+}
 
 /*
  *  stress_rand_data_parity()
@@ -1003,11 +1011,11 @@ static void stress_rand_data_latin(
 	(void)args;
 
 	if (!ptr)
-		ptr = lorem_ipsum[stress_mwc32() % SIZEOF_ARRAY(lorem_ipsum)];
+		ptr = lorem_ipsum[stress_mwc32modn(SIZEOF_ARRAY(lorem_ipsum))];
 
 	while (dataptr < end) {
 		if (!*ptr)
-			ptr = lorem_ipsum[stress_mwc32() % SIZEOF_ARRAY(lorem_ipsum)];
+			ptr = lorem_ipsum[stress_mwc32modn(SIZEOF_ARRAY(lorem_ipsum))];
 
 		*dataptr++ = *ptr++;
 	}
@@ -1044,7 +1052,7 @@ static void stress_rand_data_morse(
 		register char *mptr;
 
 		if (!*ptr)
-			ptr = lorem_ipsum[stress_mwc32() % SIZEOF_ARRAY(lorem_ipsum)];
+			ptr = lorem_ipsum[stress_mwc32modn(SIZEOF_ARRAY(lorem_ipsum))];
 
 		ch = tolower((int)*ptr);
 		mptr = morse_table[ch];
@@ -1114,7 +1122,7 @@ static void stress_rand_data_objcode(
 		text_start = (char *)stress_rand_data_objcode;
 		text_end = (char *)stress_rand_data_bcd;
 	}
-	text = text_start + (stress_mwc64() % (uint64_t)(text_end - text_start));
+	text = text_start + (stress_mwc64modn((uint64_t)(text_end - text_start)));
 
 	for (dataptr = (char *)data; dataptr < (char *)data_end; dataptr++) {
 		*dataptr = *text;
@@ -1158,9 +1166,10 @@ static const stress_zlib_rand_data_info_t zlib_rand_data_methods[] = {
 	{ "binary",	stress_rand_data_binary },
 	{ "brown",	stress_rand_data_brown },
 	{ "double",	stress_rand_data_double },
+	{ "fixed",	stress_rand_data_fixed },
 	{ "gcr",	stress_rand_data_gcr },
 	{ "gray",	stress_rand_data_gray },
-	{ "fixed",	stress_rand_data_fixed },
+	{ "inc16",	stress_rand_data_inc16 },
 	{ "latin",	stress_rand_data_latin },
 #if defined(HAVE_INT128_T)
 	{ "lehmer",	stress_rand_data_lehmer },
@@ -1178,8 +1187,7 @@ static const stress_zlib_rand_data_info_t zlib_rand_data_methods[] = {
 	{ "pink",	stress_rand_data_pink },
 	{ "rarely1",	stress_rand_data_rarely_1 },
 	{ "rarely0",	stress_rand_data_rarely_0 },
-#if defined(HAVE_ASM_X86_RDRAND) &&		\
-    defined(STRESS_ARCH_X86_64)
+#if defined(HAVE_ASM_X86_RDRAND)
 	{ "rdrand",	stress_rand_data_rdrand },
 #endif
 	{ "ror32",	stress_rand_data_ror32 },
@@ -1200,7 +1208,7 @@ static void stress_zlib_random_test(
 {
 	/* We ignore 1st method (random) and last (NULL) entry */
 	const int max = SIZEOF_ARRAY(zlib_rand_data_methods) - 2;
-	const int idx = (stress_mwc32() % max) + 1;
+	const int idx = (stress_mwc32modn(max)) + 1;
 
 	zlib_rand_data_methods[idx].func(args, data, data_end);
 }
@@ -1523,7 +1531,7 @@ static int stress_zlib_deflate(
 	uint64_t bytes_in = 0, bytes_out = 0;
 	int flush;
 	stress_zlib_args_t zlib_args;
-	double t1, t2;
+	double t1, duration, rate, ratio;
 	stress_zlib_checksum_t zlib_checksum;
 	stress_zlib_rand_data_info_t *info;
 
@@ -1670,11 +1678,12 @@ static int stress_zlib_deflate(
 	} while (keep_stressing(args));
 
 finish:
-	t2 = stress_time_now();
-	pr_inf("%s: instance %" PRIu32 ": compression ratio: %5.2f%% (%.2f MB/sec)\n",
-		args->name, args->instance,
-		bytes_in ? 100.0 * (double)bytes_out / (double)bytes_in : 0,
-		(t2 - t1 > 0.0) ? ((double)bytes_in / (t2 - t1)) / MB : 0.0);
+	duration = stress_time_now() - t1;
+
+	ratio = (bytes_in > 0) ? 100.0 * (double)bytes_out / (double)bytes_in : 0.0;
+	stress_metrics_set(args, 0, "% compression ratio", ratio);
+	rate = (duration > 0.0) ? ((double)bytes_in / duration) / MB : 0.0;
+	stress_metrics_set(args, 1, "MB/sec compression rate", rate);
 
 	ret = EXIT_SUCCESS;
 zlib_checksum_error:
@@ -1864,8 +1873,9 @@ stressor_info_t stress_zlib_info = {
 };
 #else
 stressor_info_t stress_zlib_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without zlib library support"
 };
 #endif
